# Rekall Memory Forensics
# Copyright (C) 2012 Michael Cohen
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module implements the rekall session.

The session stores information about the a specific user interactive
session. Sessions can be saved and loaded between runs and provide a convenient
way for people to save their own results.
"""

__author__ = "Michael Cohen <scudette@gmail.com>"

import inspect
import json
import logging
import pdb
import os
import sys
import time

from rekall import addrspace
from rekall import config
# Include the built in profiles as a last fallback.
from rekall import io_manager
from rekall import plugin
from rekall import obj
from rekall import utils
from rekall.ui import renderer


# Top level args.
config.DeclareOption("-p", "--profile",
                     help="Name of the profile to load. This is the "
                     "filename of the profile found in the profiles "
                     "directory. Profiles are searched in the profile "
                     "path order.")

config.DeclareOption(
    "--profile_path", default=[], action="append",
    help="Path to search for profiles. This can take "
    "any form supported by the IO Manager (e.g. zip files, "
    "directories, URLs etc)")


config.DeclareOption("-f", "--filename",
                     help="The raw image to load.")


class Container(object):
    """Just a container."""


class Cache(utils.AttributeDict):

    def _CheckCorrectType(self, value):
        """Ensure that the configuration remains json serializable."""
        if value is None:
            return True

        if isinstance(value, (int, long, basestring, float)):
            return True

        if isinstance(value, (list, tuple)):
            return all((self._CheckCorrectType(x) for x in value))

        if isinstance(value, dict):
            return (self._CheckCorrectType(value.keys()) and
                    self._CheckCorrectType(value.values()))

        return False

    def __str__(self):
        return json.dumps(self, indent=2, sort_keys=True)

    def __repr__(self):
        return "<Configuration Object>"

    def _set_filename(self, filename):
        if filename:
            self['filename'] = filename
            self['base_filename'] = os.path.basename(filename)

    def _set_logging(self, value):
        level = value
        if isinstance(value, basestring):
            level = getattr(logging, value.upper(), logging.INFO)

        if level is None:
            return

        logging.info("Logging level set to %s", value)
        logging.getLogger().setLevel(int(level))

    def Set(self, attr, value):
        hook = getattr(self, "_set_%s" % attr, None)
        if hook:
            hook(value)

        else:
            if not self._CheckCorrectType(value):
                raise ValueError(
                    "Configuration parameters must be simple types, not %r." %
                    value)

            super(Cache, self).Set(attr, value)


class Configuration(Cache):
    # The session which owns this configuration object.
    session = None

    # This holds a write lock on the configuration object.
    _lock = False

    def __init__(self, session=None, **kwargs):
        super(Configuration, self).__init__(**kwargs)
        self.session = session
        self.update(**kwargs)

        # Can not update the configuration object any more.
        self._lock = True

    def Set(self, attr, value):
        if self._lock:
            raise ValueError(
                "Can only update configuration using the context manager.")

        super(Configuration, self).Set(attr, value)

    def __enter__(self):
        # Allow us to update the context manager.
        self._lock = False
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._lock = True
        if self.session:
            self.session.UpdateFromConfigObject()



class Session(object):
    """Base session.

    This session contains the bare minimum to use rekall.
    """
    def __init__(self, **kwargs):
        self.profile = obj.NoneObject("Set this to a valid profile "
                                      "(e.g. type profiles. and tab).")

        # Store user configurable attributes here. These will be read/written to
        # the configuration file.
        self.state = Configuration(self, cache=Cache(), **kwargs)
        self.UpdateFromConfigObject()

    def UpdateFromConfigObject(self):
        """This method is called whenever the config object was updated.

        We are expected to re-check the config and re-initialize this session.
        """
        self.filename = self.state.filename
        if self.state.profile:
            self.profile = self.LoadProfile(self.state.profile)

        # Set the renderer.
        self.renderer = renderer.RendererBaseClass.classes.get(
            self.GetParameter("renderer"), "TextRenderer")

        self._update_runners()

    def _update_runners(self):
        plugins = Container()
        object.__setattr__(self, "plugins", plugins)
        for cls in plugin.Command.GetActiveClasses(self):
            name = cls.name
            if name:
                setattr(plugins, name, obj.Curry(cls, session=self))

    def __getattr__(self, attr):
        """This will only get called if the attribute does not exist."""
        return None

    def GetParameter(self, item, default=None):
        """Retrieves a stored parameter.

        Parameters are managed by the Rekall session in two layers. The state
        containers contains those parameters which are deliberately set by the
        user.

        Some parameters are calculated by plugins and are used in order to speed
        up further calculations. These are cached in the state as well.

        It is important to never override a user selection by the cached
        results. Since the user must be allowed to override all parameters - for
        example through the GUI or the command line. Therefore when resolving a
        parameter, we first check in the state, and only if the parameter does
        not exist, we check the cache.
        """
        result = self.state.Get(item)
        if result is None:
            result = self.state.cache.Get(item)

        if result is None:
            result = default

        return result

    def StoreParameter(self, item, value):
        self.state.cache[item] = value

    def error(self, _plugin_cls, e):
        """An error handler for plugin errors."""
        raise e

    def RunPlugin(self, plugin_cls, *pos_args, **kwargs):
        """Launch a plugin and its render() method automatically.

        We use the pager specified in session.state.pager.

        Args:
          plugin_cls: A string naming the plugin, or the plugin class itself.
          renderer: An optional renderer to use.
          debug: If set we break into the debugger if anything goes wrong.

          output: If set we open and write the output to this filename. If
            session.overwrite is set to True, we will overwrite this
            file. Otherwise the output is redirected to stdout.
        """
        ui_renderer = kwargs.pop("renderer", None)
        fd = kwargs.pop("fd", None)
        debug = kwargs.pop("debug", False)
        output = kwargs.pop("output", None)
        overwrite = kwargs.get("overwrite")
        flags = kwargs.get("flags")

        if isinstance(plugin_cls, basestring):
            plugin_name = plugin_cls
            plugin_cls = getattr(self.plugins, plugin_cls, None)
            if plugin_cls is None:
                logging.error("Plugin %s is not active. Is it supported with "
                              "this profile?", plugin_name)
                return

        # If the args came from the command line parse them now:
        if flags:
            from rekall import args

            kwargs = args.MockArgParser().build_args_dict(plugin_cls, flags)

        # Select the renderer from the session or from the kwargs.
        if not isinstance(ui_renderer, renderer.RendererBaseClass):
            ui_renderer_cls = self.renderer or renderer.TextRenderer

            if isinstance(ui_renderer_cls, basestring):
                ui_renderer_cls = renderer.TextRenderer.classes[ui_renderer_cls]

            # Allow the output to be written to file.
            if output is not None:
                if os.access(output, os.F_OK) and not (
                    overwrite or self.overwrite):
                    logging.error(
                        "Output file '%s' exists but session.overwrite is "
                        "not set." % output)
                    return
                else:
                    fd = open(output, "w")

            # Allow per call overriding of the output file descriptor.
            paging_limit = self.state.paging_limit
            if not self.state.pager:
                paging_limit = None

            ui_renderer = ui_renderer_cls(session=self, fd=fd,
                                          paging_limit=paging_limit)

        try:
            kwargs['session'] = self

            ui_renderer.start()

            # If we were passed an instance we do not instantiate it.
            if inspect.isclass(plugin_cls) or isinstance(plugin_cls, obj.Curry):
                result = plugin_cls(*pos_args, **kwargs)
            else:
                result = plugin_cls

            ui_renderer.start(plugin_name=result.name, kwargs=kwargs)

            try:
                result.render(ui_renderer)
            finally:
                ui_renderer.end()

            # If there was too much data and a pager is specified, simply pass
            # the data to the pager:
            if (self.state.pager and
                len(ui_renderer.data) >= self.state.paging_limit):
                pager = renderer.Pager(self)
                for data in ui_renderer.data:
                    pager.write(data)

                # Now wait for the user to exit the pager.
                pager.flush()

            return result

        except plugin.InvalidArgs, e:
            logging.error("Invalid Args (Try info plugins.%s): %s",
                          plugin_cls.name, e)

        except plugin.Error, e:
            self.error(plugin_cls, e)

        except KeyboardInterrupt:
            if self.debug:
                pdb.post_mortem()

            self.report_progress("Aborted!\r\n", force=True)

        except Exception, e:
            logging.error("Error: %s", e)
            # If anything goes wrong, we break into a debugger here.
            if debug:
                pdb.post_mortem()
            else:
                raise

    def LoadProfile(self, filename):
        """Try to load a profile directly from a filename.

        Args:
          filename: A string which will be used to get an io_manager
            container. If it contains a path sepearator we open the file
            directly, otherwise we search in the profile_path specification.

        Returns:
          a Profile() instance or a NoneObject()
        """
        if not filename:
            return

        # We only want to deal with unix paths.
        filename = filename.replace("\\", "/")
        canonical_name = os.path.splitext(os.path.basename(filename))[0]

        # The filename is a path we try to open it directly:
        if "/" in filename:
            result = obj.Profile.LoadProfileFromContainer(
                io_manager.Factory(filename), self, name=canonical_name)

        # Traverse the profile path until one works.
        else:
            result = None
            # The profile path is specified in search order.
            profile_path = self.state.Get("profile_path")

            # Make sure that we always fallback to the built in profiles last.
            if None not in profile_path:
                profile_path.append(None)

            for path in profile_path:
                try:
                    manager = io_manager.Factory(path)
                    result = obj.Profile.LoadProfileFromContainer(
                        manager.OpenSubContainer(filename), self,
                        name=canonical_name)
                    logging.info("Loaded profile %s from %s",
                                 filename, manager)
                    break

                except (IOError, KeyError) as e:
                    result = obj.NoneObject(e)
                    logging.debug("Could not find profile %s in %s",
                                  filename, path)

                    continue

        if not result:
            raise ValueError("Container %s is not a valid rekall profile." %
                             filename)

        return result

    def __unicode__(self):
        return u"Session"

    def report_progress(self, message="", *args, **kwargs):
        """Called by the library to report back on the progress."""
        if callable(self.progress):
            self.progress(message, *args, **kwargs)


class InteractiveSession(Session):
    """The session allows for storing of arbitrary values and configuration.

    This session contains a lot of convenience features which are useful for
    interactive use.
    """

    def __init__(self, env=None, **kwargs):
        self._locals = env or {}

        # These are the command plugins which we exported to the local
        # namespace.
        self._start_time = time.time()

        # These keep track of the last run plugin.
        self._last_plugin = None

        # Fill the session with helpful defaults.
        self.pager = obj.NoneObject("Set this to your favourite pager.")

        super(InteractiveSession, self).__init__(**kwargs)

    def _update_runners(self):
        plugins = Container()
        self._locals['plugins'] = Container()

        object.__setattr__(self, "plugins", plugins)
        for cls in plugin.Command.GetActiveClasses(self):
            name = cls.name
            if name:
                setattr(plugins, name, obj.Curry(cls, session=self))

                # Use the info class to build docstrings for all plugins.
                info_plugin = plugin.Command.classes['Info'](cls)

                # Create a runner for this plugin and set its documentation.
                runner = obj.Curry(
                    self.RunPlugin, name, default_arguments=[
                        x for x, _ in info_plugin.get_default_args()])

                runner.__doc__ = utils.SmartUnicode(info_plugin)

                setattr(self._locals['plugins'], name, runner)
                self._locals[name] = runner

    def reset(self):
        """Reset the current session by making a new session."""
        self._prepare_local_namespace()

    def RunPlugin(self, *args, **kwargs):
        self.last = super(InteractiveSession, self).RunPlugin(*args, **kwargs)

    def _prepare_local_namespace(self):
        session = self._locals['session'] = self
        # Prepopulate the namespace with our most important modules.
        self._locals['addrspace'] = addrspace
        self._locals['obj'] = obj
        self._locals['profile'] = self.profile
        self._locals['v'] = session.v

        # Add all plugins to the local namespace and to their own container.
        self._update_runners()

        # Some useful modules which should be available always.
        self._locals["sys"] = sys
        self._locals["os"] = os

    def v(self):
        """Re-execute the previous command."""
        if self.last:
            self.RunPlugin(self.last)

    def lister(self, arg):
        for x in arg:
            self.printer(x)

    def __str__(self):
        result = """Rekall Memory Forensics session Started on %s.

Config:
%s
""" % (time.ctime(self.start_time), self.state)
        return result

    def __dir__(self):
        items = self.__dict__.keys() + dir(self.__class__)

        return [x for x in items if not x.startswith("_")]

    def error(self, plugin_cls, e):
        """Swallow the error but report it."""
        logging.error("Failed running plugin %s: %s",
                      plugin_cls.name, e)



# Rekall Memory Forensics
# Copyright (C) 2012 Michael Cohen
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module print details information about PE files and processes.

Output is similar to objdump or pefile.
"""

__author__ = "Michael Cohen <scudette@gmail.com>"

from rekall import args
from rekall import plugin
from rekall import testlib

from rekall.plugins.addrspaces import standard
from rekall.plugins.overlays.windows import pe_vtypes
from rekall.plugins.windows import common


class PEInfo(plugin.Command):
    """Print information about a PE binary."""

    __name = "peinfo"

    @classmethod
    def args(cls, parser):
        super(PEInfo, cls).args(parser)
        parser.add_argument("--image_base", default=0, type=args.IntParser,
                            help="The base of the image.")

        parser.add_argument("--filename", default=None,
                            help="If provided we create an address space "
                            "from this file.")


    def __init__(self, address_space=None, image_base=0, filename=None,
                 **kwargs):
        """Dump a PE binary from memory.

        Status is shown for each exported function:

          - M: The function is mapped into memory.

        Args:
          address_space: The address space which contains the PE image.
          image_base: The address of the image base (dos header).
          filename: If provided we create an address space from this file.
        """
        super(PEInfo, self).__init__(**kwargs)
        self.address_space = address_space
        if filename:
            self.address_space = standard.FileAddressSpace(filename=filename)
        self.image_base = image_base

    def render(self, renderer):
        """Print information about a PE file from memory."""
        try:
            disassembler = self.session.plugins.dis(
                address_space=self.address_space, offset=0,
                session=self.session, length=50)
        except AttributeError:
            disassembler = None

        # Get our helper object to parse the PE file.
        pe_helper = pe_vtypes.PE(address_space=self.address_space,
                                 image_base=self.image_base)

        renderer.table_header([('Machine', 'machine', '<20'),
                               ('TimeDateStamp', 'time', '[wrap:60]')])

        for field in ["Machine", "TimeDateStamp", "Characteristics"]:
            renderer.table_row(field,
                               getattr(pe_helper.nt_header.FileHeader, field))

        renderer.format("\nSections (Relative to 0x{0:08X}):\n",
                        pe_helper.image_base)
        renderer.table_header([('Perm', 'perm', '4'),
                               ('Name', 'name', '<8'),
                               ('VMA', 'vma', '[addrpad]'),
                               ('Size', 'size', '[addrpad]')])

        for permission, name, virtual_address, size in pe_helper.Sections():
            renderer.table_row(permission, name, virtual_address, size)

        renderer.format("\nData Directories:\n")
        renderer.table_header([('', 'name', '<40'),
                               ('VMA', 'vma', '[addrpad]'),
                               ('Size', 'size', '[addrpad]')])

        for d in pe_helper.nt_header.OptionalHeader.DataDirectory:
            renderer.table_row(d.obj_name, d.VirtualAddress, d.Size)


        renderer.format("\nImport Directory (Original):\n")
        renderer.table_header([('Name', 'name', '<50'),
                               ('Ord', 'ord', '5')])

        for dll, name, ordinal in pe_helper.ImportDirectory():
            renderer.table_row(u"%s!%s" % (dll, name), ordinal)

        renderer.format("\nImport Address Table:\n")
        renderer.table_header([('Name', 'name', '<20'),
                               ('Address', 'address', '[addrpad]'),
                               ('Disassembly', 'disassembly', '[wrap:30]')])

        for name, function, ordinal in pe_helper.IAT():
            disassembly = []

            if disassembler:
                for i, (_, _, x) in enumerate(
                    disassembler.disassemble(function)):
                    if i >= 5:
                        break

                    disassembly.append(x.strip())

            renderer.table_row(name, function, "\n".join(disassembly))

        renderer.format("\nExport Directory:\n")
        renderer.table_header([('Entry', 'entry', '[addrpad]'),
                               ('Stat', 'status', '4'),
                               ('Ord', 'ord', '5'),
                               ('Name', 'name', '<50')])

        for dll, function, name, ordinal in pe_helper.ExportDirectory():
            status = 'M' if function.dereference() else "-"
            renderer.table_row(
                function,
                status,
                ordinal,
                u"%s!%s" % (dll, name))

            self.address_space.kb.AddMemoryLocation(int(function), function)

        renderer.format("Version Information:\n")
        renderer.table_header([('key', 'key', '<20'),
                               ('value', 'value', '')])

        for k, v in pe_helper.VersionInformation():
            renderer.table_row(k, v)


class ProcInfo(common.WinProcessFilter):
    """Dump detailed information about a running process."""

    __name = "procinfo"

    def render(self, renderer):
        for task in self.filter_processes():
            renderer.section()
            renderer.format("Pid: %s %s\n",
                            task.UniqueProcessId, task.ImageFileName)

            task_address_space = task.get_process_address_space()
            if not task_address_space:
                renderer.format("Peb Not mapped.\n")
                continue

            renderer.format("\nProcess Environment\n")
            # The environment is just a sentinal terminated array of strings.
            for line in task.Peb.ProcessParameters.Environment:
                renderer.format("   %s\n" % line)

            renderer.format("\nPE Infomation\n")

            # Parse the PE file of the main process's executable.
            pe = PEInfo(address_space=task_address_space,
                        session=self.session,
                        image_base=task.Peb.ImageBaseAddress)

            pe.render(renderer)


class TestProcInfo(testlib.SimpleTestCase):
    PARAMETERS = dict(
        commandline="procinfo --pid=%(pid)s"
        )



# Rekall Memory Forensics
# Copyright (c) 2012 Michael Cohen <scudette@gmail.com>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""A parser for dwarf modules which generates vtypes."""
import json
import optparse
import sys

from elftools.dwarf import descriptions
from elftools.elf import elffile


class DIETag(object):
    # The parent container for this DIE.
    parent = None

    def __init__(self, die, types, parents):
        self.attributes = {}
        self.die = die
        self.types = types
        if parents:
            self.parent = parents[-1]

        for attr in die.attributes.values():
            self.attributes[attr.name] = attr

    @property
    def name(self):
        if "DW_AT_name" in self.attributes:
            return self.attributes["DW_AT_name"].value
        else:
            return "__unnamed_%s" % self.die.offset

    @property
    def type_id(self):
        if "DW_AT_type" in self.attributes:
            return self.attributes["DW_AT_type"].value + self.die.cu.cu_offset

    def VType(self):
        """Returns a vtype representation of this DIE."""
        return self.name

    def Definition(self):
        pass

class DW_TAG_typedef(DIETag):

    def VType(self):
        """For typedefs we just substitute our base type."""
        return self.types[self.type_id].VType()


class DW_TAG_volatile_type(DW_TAG_typedef):
    pass


class DW_TAG_base_type(DIETag):
    """A base type."""


class DW_TAG_structure_type(DIETag):
    """A struct definition."""
    def __init__(self, die, types, parents):
        super(DW_TAG_structure_type, self).__init__(die, types, parents)
        # All the members of this struct.
        self.members = []

    @property
    def size(self):
        try:
            return self.attributes['DW_AT_byte_size'].value
        except KeyError:
            pass

    def Definition(self):
        count = 1
        result = [self.size, {}]
        for member in self.members:
            if isinstance(member, DW_TAG_member):
                child = member.delegate()
                name = member.name

                # Make nicer names for annonymous things (usually unions).
                if name.startswith("__unnamed_"):
                    name = "u%s" % count
                    count += 1

                result[1][name] = member.VType()

        # Only emit structs with actual members in them.
        if result[1]:
            return [self.name, result]


class DW_TAG_union_type(DW_TAG_structure_type):
    pass


class DW_TAG_pointer_type(DIETag):
    def VType(self):
        if 'DW_AT_type' in self.attributes:
            target = self.types[self.type_id]
            target_type = target.VType()
            if not isinstance(target_type, list):
                target_type = [target_type, None]

            return ['Pointer', dict(target=target_type[0],
                                    target_args=target_type[1])]

        return ['Pointer', dict(target="Void")]


class DW_TAG_subroutine_type(DIETag):
    def VType(self):
        return "void"


class DW_TAG_array_type(DIETag):
    count = 0

    def VType(self):
        if 'DW_AT_type' in self.attributes:
            target_type = self.types[self.type_id].VType()
            if not isinstance(target_type, list):
                target_type = [target_type, None]
            return ['Array', dict(target=target_type[0],
                                  target_args=target_type[1],
                                  count=self.count)]

        return ['Array', dict(count=self.count)]


class DW_TAG_subrange_type(DIETag):
    """These specify the count of arrays."""
    def __init__(self, die, types, parents):
        super(DW_TAG_subrange_type, self).__init__(die, types, parents)

        if "DW_AT_upper_bound" in self.attributes:
            self.parent.count = self.attributes['DW_AT_upper_bound'].value + 1


_DWARF_EXPR_DUMPER_CACHE = {}
def describe_DWARF_expr(expr, structs):
    """ Textual description of a DWARF expression encoded in 'expr'.
        structs should come from the entity encompassing the expression - it's
        needed to be able to parse it correctly.
    """
    # Since this function can be called a lot, initializing a fresh new
    # ExprDumper per call is expensive. So a rudimentary caching scheme is in
    # place to create only one such dumper per instance of structs.
    cache_key = id(structs)
    if cache_key not in _DWARF_EXPR_DUMPER_CACHE:
        _DWARF_EXPR_DUMPER_CACHE[cache_key] = \
            descriptions.ExprDumper(structs)
    dwarf_expr_dumper = _DWARF_EXPR_DUMPER_CACHE[cache_key]
    dwarf_expr_dumper.clear()
    dwarf_expr_dumper.process_expr(expr)
    return dwarf_expr_dumper.get_str().split(":")


class DW_TAG_member(DIETag):
    offset = 0
    type = None

    def __init__(self, die, types, parents):
        super(DW_TAG_member, self).__init__(die, types, parents)

        # Add ourselves to our parent struct.
        self.parent.members.append(self)

        if 'DW_AT_data_member_location' in self.attributes:
            value = self.attributes['DW_AT_data_member_location'].value
            if isinstance(value, (int, long)):
                self.offset = value
            else:
                op_code, value = describe_DWARF_expr(value, die.cu.structs)

                if op_code=="DW_OP_plus_uconst":
                    self.offset = int(value)

    def delegate(self):
        """A member is just a place holder for another type in the struct.

        Thie method returns the delegate.
        """
        return self.types[self.type_id]

    def VType(self):
        if self.type_id not in self.types:
            import pdb; pdb.set_trace()

        member_type = self.delegate().VType()

        # Does this member represent a bitfield?
        if "DW_AT_bit_size" in self.attributes:
            # The dwarf standard says:

            # For a DW_AT_data_bit_offset attribute, the value is an integer
            # constant (see Section 2.19) that specifies the number of bits from
            # the beginning of the containing entity to the beginning of the data
            # member.

            # This means that for little endian we need to swap them with the
            # size of the integer.
            full_size = self.attributes['DW_AT_byte_size'].value * 8
            start_bit = self.attributes['DW_AT_bit_offset'].value
            end_bit = self.attributes['DW_AT_bit_size'].value + start_bit

            converted_start_bit = full_size - end_bit
            converted_end_bit = full_size - start_bit

            # Sometimes the member is an Enumeration. In that case we need to
            # return something slightly different.
            if member_type[0] == "Enumeration":
                member_type[1]["target"] = "BitField"
                member_type[1]["target_args"] = {
                    'start_bit': converted_start_bit,
                    'end_bit': converted_end_bit
                    }
                return [self.offset, member_type]

            return [self.offset, ['BitField', {'start_bit': converted_start_bit,
                                               'native_type': member_type,
                                               'end_bit': converted_end_bit}]]

        if not isinstance(member_type, list):
            member_type = [member_type]

        return [self.offset, member_type]


class DW_TAG_enumeration_type(DIETag):
    """Holds enumerations."""

    byte_size_lookup = { 4: "long",
                         2: "short int",
                         1: "char" }

    def __init__(self, die, types, parents):
        super(DW_TAG_enumeration_type, self).__init__(die, types, parents)
        self.enumerations = {}

    def VType(self):
        byte_size = self.attributes['DW_AT_byte_size'].value
        return ['Enumeration', {'choices': self.enumerations,
                                'target': self.byte_size_lookup[byte_size]}]

class DW_TAG_enumerator(DIETag):
    """An enumeration."""

    def __init__(self, die, types, parents):
        super(DW_TAG_enumerator, self).__init__(die, types, parents)

        # Add ourselves to our parent container.
        value = self.attributes['DW_AT_const_value'].value
        self.parent.enumerations[value] = self.name


# A lookup table of the different tag handlers.
DIE_LOOKUP = {
    "DW_TAG_base_type": DW_TAG_base_type,
    "DW_TAG_structure_type": DW_TAG_structure_type,
    "DW_TAG_union_type": DW_TAG_union_type,
    "DW_TAG_member": DW_TAG_member,

    # typedefs and const types are just proxies to native types.
    "DW_TAG_typedef": DW_TAG_typedef,
    "DW_TAG_const_type": DW_TAG_typedef,

    # Enumrations.
    "DW_TAG_enumeration_type": DW_TAG_enumeration_type,
    "DW_TAG_enumerator": DW_TAG_enumerator,

    "DW_TAG_pointer_type": DW_TAG_pointer_type,
    "DW_TAG_array_type": DW_TAG_array_type,
    "DW_TAG_subrange_type": DW_TAG_subrange_type,
    "DW_TAG_subroutine_type": DW_TAG_subroutine_type,
    "DW_TAG_volatile_type": DW_TAG_volatile_type,
    }

def DIEFactory(die, types, parents):
    """Returns an instance of the DIE object."""
    if die.tag in DIE_LOOKUP:
        return DIE_LOOKUP[die.tag](die, types, parents)

    return DIETag(die, types, parents)


class DWARFParser(object):
    """A parser for DWARF files."""

    def __init__(self, fd):
        self.elffile = elffile.ELFFile(fd)
        self.types = {}

        if self.elffile.has_dwarf_info():
            self._dwarfinfo = self.elffile.get_dwarf_info()
        else:
            raise RuntimeError("File does not have DWARF information - "
                               "was it compiled with debugging information?")
        self.compile()

    def compile(self):
        """Compile the vtypes from the dwarf information."""
        # We currently dump all compilation units into the same
        # vtype. Technically the same symbol can be defined differently in
        # different compilation units, but rekall does not have CU
        # resolution right now so we assume they are all the same.
        parents = []
        for cu in self._dwarfinfo.iter_CUs():
            parents.append(cu)

            for die in cu.iter_DIEs():
                if die.is_null():
                    parents = parents[:-1]
                    continue

                # Record the type in this DIE.
                t = self.types[die.offset] = DIEFactory(die, self.types, parents)

                if die.has_children:
                    parents.append(t)

    def VType(self):
        """Build a vtype for this module's dwarf information."""
        result = {}
        for type_id, type in self.types.items():
            # Only structs emit definitions basically.
            definition = type.Definition()
            if definition:
                result[definition[0]] = definition[1]

        return result


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print "Usage: dwarfparser.py  module.ko > module.json"
    else:
        with open(sys.argv[1], "rb") as fd:
            parser = DWARFParser(fd)
            vtypes = parser.VType()

            print json.dumps(vtypes, indent=1)



# Rekall Memory Forensics
# Copyright (c) 2008-2011 Volatile Systems
# Copyright 2013 Google Inc. All Rights Reserved.

# Author: Michael Cohen <scudette@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

import windows

from rekall import obj
from rekall.plugins.overlays import basic
from rekall.plugins.overlays.windows import windows
from rekall.plugins.overlays.windows import win7


# In windows 8 the VadRoot is actually composed from _MM_AVL_NODE instead of
# _MMVAD structs or _MMADDRESS_NODE. The structs appear to be organised by
# functional order - the _MM_AVL_NODE is always the first member of the struct,
# then the _MMVAD_SHORT, then the _MMVAD. For example, the tree traversal code,
# simply casts all objects to an _MM_AVL_NODE without caring what they actually
# are, then depending on the vad tag, they get casted to different structs.
win8_overlays = {
    '_EPROCESS': [ None, {
            # A symbolic link to the real vad root.
            'RealVadRoot': lambda x: x.VadRoot.BalancedRoot
            }],

    '_MM_AVL_NODE': [ None, {
            'Tag': [-12 , ['String', dict(length = 4)]],
            }],

    '_MMVAD_SHORT': [ None, {
            'Tag': [-12 , ['String', dict(length = 4)]],
            'Start': lambda x: x.StartingVpn << 12,
            'End': lambda x: ((x.EndingVpn + 1) << 12) - 1,
            'CommitCharge': lambda x: x.u1.VadFlags1.CommitCharge,
            }],

    '_MMVAD': [ None, {
            'Tag': [-12 , ['String', dict(length = 4)]],
            'ControlArea': lambda x: x.Subsection.ControlArea,
            'Start': lambda x: x.Core.StartingVpn << 12,
            'End': lambda x: ((x.Core.EndingVpn + 1) << 12) - 1,
            'CommitCharge': lambda x: x.Core.u1.VadFlags1.CommitCharge,
            'u': lambda x: x.Core.u,
            }],

    "_CONTROL_AREA": [None, {
            'FilePointer': [None, ['_EX_FAST_REF', dict(target="_FILE_OBJECT")]],
            }],
    }


class _OBJECT_HEADER(win7._OBJECT_HEADER):
    """A Rekall Memory Forensics object to handle Windows 7 object headers.

    Windows 7 changes the way objects are handled:
    References: http://www.codemachine.com/article_objectheader.html
    """

    type_map = { 2: 'Type',
                3: 'Directory',
                4: 'SymbolicLink',
                5: 'Token',
                6: 'Job',
                7: 'Process',
                8: 'Thread',
                9: 'UserApcReserve',
                10: 'IoCompletionReserve',
                11: 'DebugObject',
                12: 'Event',
                13: 'EventPair',
                14: 'Mutant',
                15: 'Callback',
                16: 'Semaphore',
                17: 'Timer',
                18: 'Profile',
                19: 'KeyedEvent',
                20: 'WindowStation',
                21: 'Desktop',
                22: 'TpWorkerFactory',
                23: 'Adapter',
                24: 'Controller',
                25: 'Device',
                26: 'Driver',
                27: 'IoCompletion',
                28: 'File',
                29: 'TmTm',
                30: 'TmTx',
                31: 'TmRm',
                32: 'TmEn',
                33: 'Section',
                34: 'Session',
                35: 'Key',
                36: 'ALPC Port',
                37: 'PowerRequest',
                38: 'WmiGuid',
                39: 'EtwRegistration',
                40: 'EtwConsumer',
                41: 'FilterConnectionPort',
                42: 'FilterCommunicationPort',
                43: 'PcwObject',
            }

    # This specifies the order the headers are found below the _OBJECT_HEADER
    optional_header_mask = (('CreatorInfo', '_OBJECT_HEADER_CREATOR_INFO', 0x01),
                            ('NameInfo', '_OBJECT_HEADER_NAME_INFO', 0x02),
                            ('HandleInfo', '_OBJECT_HEADER_HANDLE_INFO', 0x04),
                            ('QuotaInfo', '_OBJECT_HEADER_QUOTA_INFO', 0x08),
                            ('ProcessInfo', '_OBJECT_HEADER_PROCESS_INFO', 0x10),
                            ('AuditInfo', '_OBJECT_HEADER_AUDIT_INFO', 0x40),
                            )

    def find_optional_headers(self):
        """Find this object's optional headers."""
        offset = self.obj_offset

        info_mask = int(self.InfoMask)

        for name, struct, mask in self.optional_header_mask:
            if info_mask & mask:
                offset -= self.obj_profile.get_obj_size(struct)
                o = self.obj_profile.Object(type_name=struct, offset=offset,
                                            vm=self.obj_vm)
                self._preamble_size += o.size()
            else:
                o = obj.NoneObject("Header not set")

            setattr(self,name, o)

    def get_object_type(self, kernel_address_space):
        """Return the object's type as a string"""
        return self.type_map.get(self.TypeIndex.v(), '')

    def is_valid(self):
        """Determine if the object makes sense."""
        # These need to be reasonable.
        pointer_count = int(self.PointerCount)
        if pointer_count > 0x100000 or pointer_count < 0:
            return False

        handle_count = int(self.HandleCount)
        if handle_count > 0x1000 or handle_count < 0:
            return False

        if  (self.TypeIndex >= len(self.type_map) or
             self.TypeIndex < 1):
            return False

        return True


class _HANDLE_TABLE(obj.Struct):
    @property
    def HandleCount(self):
        # We dont know how to figure this out yet!
        return 0



class _POOL_HEADER(obj.Struct):
    MAX_PREAMBLE_SIZE = 0xa0

    def get_next_object(self, offset, object_name):
        """Gets the next object that fits after this offset."""
        pool_align = self.obj_profile.get_constant("PoolAlignment")
        for preamble in range(0, self.MAX_PREAMBLE_SIZE, pool_align):
            result = self.obj_profile.Object(object_name, vm=self.obj_vm,
                                             offset=offset + preamble)

            # If the object preamble is exactly what we expect, we found it.
            if result.preamble_size() == preamble:
                try:
                    # Allow the object to check for its own validity.
                    if result.is_valid():
                        return result
                except AttributeError:
                    return result

    def get_object(self, object_name, allocations):
        """On windows 8, pool allocations are done from preset sizes. This means
        that the allocation is never exactly the same size and we can not use
        the bottom up method like before.

        We therefore, have to build the headers forward by checking the preamble
        size and validity of each object. This is a little slower than with
        earlier versions of windows.
        """
        objects = []

        offset = self.obj_offset

        for name in allocations:
            result = self.get_next_object(offset, name)
            if not result:
                return obj.NoneObject("Object not found.")

            offset = result.obj_offset + result.size()
            objects.append(result)

            # Return to the caller.
            if name == object_name:
                return result

        raise KeyError("object not present in preamble.")


class _MM_AVL_NODE(win7._MMADDRESS_NODE):
    """All nodes in the Vad tree are treated as _MM_AVL_NODE.

    The Vad structures can be either _MMVAD_SHORT or _MMVAD. At the
    base of each struct there is an _MM_AVL_NODE which contains the LeftChild
    and RightChild members. In order to traverse the tree, we follow the
    _MM_AVL_NODE and create the required _MMVAD type at each point.

    In Windows 8 these behave the same as windows 7's _MMADDRESS_NODE.
    """

    ## The actual type depends on this tag value. Windows 8 does not have an
    ## _MMVAD_LONG.
    tag_map = {'Vadl': '_MMVAD',
               'VadS': '_MMVAD_SHORT',
               'Vad ': '_MMVAD',
               'VadF': '_MMVAD_SHORT',
               'Vadm': '_MMVAD',
              }


class Win8BaseProfile(windows.BaseWindowsProfile):
    """The common ancestor of all windows 7 profiles."""

    __abstract = True

    def __init__(self, **kwargs):
        super(Win8BaseProfile, self).__init__(**kwargs)
        self.add_types({
                'pointer64': ['NativeType', dict(format_string='<Q')]
                })
        self.add_overlay(win8_overlays)

        self.add_classes(dict(_OBJECT_HEADER=_OBJECT_HEADER,
                              _HANDLE_TABLE=_HANDLE_TABLE,
                              _MM_AVL_NODE=_MM_AVL_NODE,
                              pointer64=obj.Pointer))


class Win8SP0x86(basic.Profile32Bits, Win8BaseProfile):
    """ A Profile for Windows 8 SP0 x86 """
    __abstract = True   # Not implemented yet.

    _md_major = 6
    _md_minor = 2


class Win8SP0x64(basic.ProfileLLP64, Win8BaseProfile):
    """ A Profile for Windows 8 SP0 x64 """
    _md_major = 6
    _md_minor = 2

    def __init__(self, **kwargs):
        super(Win8SP0x64, self).__init__(**kwargs)

        # Windows 8 changes many of the pool tags.
        self.add_constants(EPROCESS_POOLTAG="Proc",
                           DRIVER_POOLTAG="Driv",
                           )

        self.add_classes(dict(_POOL_HEADER=_POOL_HEADER))



# Overlays for parsing NTFS structures.
import array
import struct

from rekall import addrspace
from rekall import args
from rekall import plugin
from rekall import obj
from rekall.plugins.overlays import basic


class Error(Exception):
    pass


class ParseError(Error):
    pass


class NTFSParseError(ParseError):
    pass


FILE_FLAGS = dict(
    READ_ONLY=0x0001,
    HIDDEN=0x0002,
    SYSTEM=0x0004,
    ARCHIVE=0x0020,
    DEVICE=0x0040,
    NORMAL=0x0080,
    TEMPORARY=0x0100,
    SPARSE=0x0200,
    REPARSE_POINT=0x0400,
    COMPRESSED=0x0800,
    OFFLINE=0x1000,
    NOT_INDEXED=0x2000,
    ENCRYPTED=0x4000
    )

ntfs_vtypes = {
    'NTFS_BOOT_SECTOR': [512, {
            "oemname": [3, ["String", dict(length=8)]],
            "sector_size": [11, ["unsigned short"]],

            "_cluster_size": [13, ["unsigned char"]],

            # The cluster_size in bytes.
            "cluster_size": lambda x: x.m("_cluster_size") * x.sector_size,

            # The total number of clusters in the volume
            "block_count": lambda x: x.m("_volume_size") / x.cluster_size,

            "_volume_size":   [40, ["unsigned long"]],
            "_mft_cluster":   [48, ["unsigned long"]],
            "_mirror_mft_cluster":   [56, ["unsigned long"]],
            "_mft_record_size": [64, ["unsigned char"]],
            "index_record_size": [68, ["unsigned char"]],
            "serial": [72, ["String", dict(length=8)]],

            # Should be 0xAA55
            "magic": [510, ["unsigned short"]],

            # The MFT can actually be fragmented so this does not have to be the
            # complete MFT.
            "MFT": [lambda x: x.m("_mft_cluster") * x.cluster_size,
                    ["Array", dict(
                        target="MFT_ENTRY",
                        target_size=lambda x: x.mft_record_size)
                     ]],
            }],

    "MFT_ENTRY": [None, {
            "magic": [0, ["String", dict(length=4)]],
            "fixup_offset": [4, ["unsigned short"]],
            "fixup_count":  [6, ["unsigned short"]],
            "logfile_sequence_number": [8, ["unsigned long long"]],
            "sequence_value": [16, ["unsigned short"]],
            "link_count": [18, ["unsigned short"]],
            "attribute_offset": [20, ["unsigned short"]],
            "flags": [22, ["Flags", dict(
                        target="unsigned short",
                        bitmap=dict(
                            ALLOCATED=0,
                            DIRECTORY=1)
                        )]],
            "mft_entry_size": [24, ["unsigned short"]],
            "mft_entry_allocated": [28, ["unsigned short"]],
            "base_record_reference": [32, ["unsigned long long"]],
            "next_attribute_id": [40, ["unsigned short"]],

            # These are fixups.
            "fixup_magic": [lambda x: x.obj_offset + x.fixup_offset,
                            ["String", dict(length=2, term=None)]],

            "fixup_table": [lambda x: x.obj_offset + x.fixup_offset + 2,
                            ["Array", dict(
                        target="String",
                        target_args=dict(length=2, term=None),
                        count=lambda x: x.fixup_count-1)]],

            # Attributes are a list of NTFS_ATTRIBUTE objects, starting from the
            # attribute_offset member.
            "_attributes": [lambda x: x.obj_offset + x.attribute_offset,
                            ["ListArray",
                             dict(target="NTFS_ATTRIBUTE",
                                  maximum_size=lambda x: x.mft_entry_size)]],
            }],

    "NTFS_ATTRIBUTE": [lambda x: x.length, {
            "type": [0, [
                    "Enumeration", dict(
                        target="unsigned int",

                        # The actual mapping between types and type names is
                        # given by the $AttrDef file. At some point we parse
                        # this file and store it in the session.
                        choices=lambda x: x.obj_profile.get_constant(
                            "ATTRIBUTE_NAMES")
                        )]],
            "length": [4, ["unsigned int"]],
            "resident": [8, ["Enumeration", dict(
                        target="unsigned char",
                        choices={
                            0: "RESIDENT",
                            1: "NON-RESIDENT",
                            }
                        )]],
            "name_length": [9, ["unsigned char"]],
            "name_offset": [10, ["unsigned short"]],
            "flags": [12, ["Flags", dict(
                        target="unsigned short",
                        maskmap={
                            "COMPRESSED" : 0x0001,
                            "ENCRYPTED": 0x4000,
                            "SPARSE": 0x8000,
                            }
                        )]],
            "attribute_id": [14, ["unsigned short"]],

            "name": [lambda x: x.obj_offset + x.name_offset,
                     ["UnicodeString", dict(
                        length=lambda x: x.name_length * 2)]],

            # The following are only valid if the attribute is resident.
            "content_size": [16, ["unsigned int"]],
            "content_offset": [20, ["unsigned short"]],

            # The following are valid if the attribute is non-resident.
            "runlist_vcn_start": [16, ["unsigned long long"]],
            "runlist_vcn_end": [24, ["unsigned long long"]],
            "runlist_offset": [32, ["unsigned short"]],
            "compression_unit_size": [34, ["unsigned short"]],
            "allocated_size": [40, ["unsigned long long"]],
            "actual_size": [48, ["unsigned long long"]],
            "initialized_size": [56, ["unsigned long long"]],

            }],

    "STANDARD_INFORMATION": [None, {
            "create_time": [0, ["WinTimeStamp"]],
            "file_altered_time": [8, ["WinTimeStamp"]],
            "mft_altered_time": [16, ["WinTimeStamp"]],
            "file_accessed_time": [24, ["WinTimeStamp"]],
            "flags": [32, ["Flags", dict(
                        target="unsigned int",
                        bitmap=FILE_FLAGS)]],
            "max_versions": [36, ["unsigned int"]],
            "version": [40, ["unsigned int"]],
            "class_id": [44, ["unsigned int"]],
            "owner_id": [48, ["unsigned int"]],
            "sid": [52, ["unsigned int"]],
            "quota": [56, ["unsigned long long"]],
            "usn": [64, ["unsigned int"]],
            }],

    "FILE_NAME": [None, {
            "parent_mft": [0, ["unsigned long long"]],
            "created": [8, ["WinTimeStamp"]],
            "file_modified": [16, ["WinTimeStamp"]],
            "mft_modified": [24, ["WinTimeStamp"]],
            "file_accessed": [32, ["WinTimeStamp"]],
            "allocated_size": [40, ["unsigned long long"]],
            "size": [48, ["unsigned long long"]],
            "flags": [56, ["Flags", dict(
                        target="unsigned int",
                        bitmap=FILE_FLAGS)]],
            "reparse_value": [60, ["unsigned int"]],
            "_length_of_name": [64, ["byte"]],
            "name_type": [65, ["Enumeration", dict(
                        target="byte",
                        choices={
                            0: "POSIX",
                            1: "Win32",
                            2: "DOS",
                            3: "DOS+Win32"
                            })]],
            "name": [66, ["UnicodeString", dict(
                        length=lambda x: x.m("_length_of_name") * 2)]],
            }],
    }


class FixupAddressSpace(addrspace.BaseAddressSpace):
    """An address space to implement record fixup."""

    def __init__(self, fixup_magic, fixup_table, base_offset, length, **kwargs):
        super(FixupAddressSpace, self).__init__(**kwargs)
        self.as_assert(self.base is not None, "Address space must be stacked.")
        self.base_offset = base_offset

        # We read the entire region into a mutable buffer then apply the fixups.
        self.buffer = array.array("c", self.base.read(base_offset, length))
        for i, fixup_value in enumerate(fixup_table):
            fixup_offset = (i+1) * 512 - 2
            if (self.buffer[fixup_offset:fixup_offset+2].tostring() !=
                fixup_magic.v()):
                raise NTFSParseError("Fixup error")

            self.buffer[fixup_offset:fixup_offset+2] = array.array(
                "c", fixup_value.v())

    def read(self, address, length):
        buffer_offset = address - self.base_offset
        return self.buffer[buffer_offset:buffer_offset+length].tostring()


class RunListAddressSpace(addrspace.PagedReader):
    """An address space which is initialized from a runlist."""

    def __init__(self, run_list, cluster_size=None, **kwargs):
        super(RunListAddressSpace, self).__init__(**kwargs)
        self.PAGE_SIZE = cluster_size or self.session.cluster_size
        self.run_list = run_list

    def vtop(self, vaddr):
        """Convert from virtual linear address to the physical offset."""
        cluster_offset = vaddr % self.PAGE_SIZE
        cluster_number = vaddr / self.PAGE_SIZE
        return (self.run_list[cluster_number] * self.PAGE_SIZE +
                cluster_offset)



class MFT_ENTRY(obj.Struct):
    """An MFT Entry."""

    def __init__(self, **kwargs):
        super(MFT_ENTRY, self).__init__(**kwargs)

        # We implement fixup by wrapping the base address space with a fixed
        # one:
        self.obj_vm = FixupAddressSpace(fixup_magic=self.fixup_magic,
                                        fixup_table=self.fixup_table,
                                        base_offset=self.obj_offset,
                                        length=self.mft_entry_allocated,
                                        base=self.obj_vm)

    @property
    def attributes(self):
        for attribute in self._attributes:
            if attribute.type == 0xFFFFFFFF:
                break

            yield attribute


class NTFS_BOOT_SECTOR(obj.Struct):
    """A class to parse and access the NTFS boot sector."""

    # The mft_record_size in bytes
    mft_record_size = 0

    def __init__(self, **kwargs):
        """Parse the boot sector and calculate offsets."""
        super(NTFS_BOOT_SECTOR, self).__init__(**kwargs)
        if self._mft_record_size > 0:
            self.mft_record_size = self._mft_record_size * self.cluster_size
        else:
            self.mft_record_size = 1 << -self._mft_record_size

    def Validate(self):
        """Verify the boot sector for sanity."""

        if self.magic != 0xAA55:
            raise NTFSParseError("Magic not correct.")

        if self.cluster_size not in [
            0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80]:
            raise NTFSParseError("Invalid cluster_size")

        if self.sector_size == 0 or self.sector_size % 512:
            raise NTFSParseError("invalid sector_size")

        if self.block_count == 0:
            raise NTFSParseError("Volume size is 0")


class NTFS_ATTRIBUTE(obj.Struct):
    """The NTFS attribute."""

    # A quick lookup to decode the runlist. Key is the byte size, value is a
    # mask to use.
    MASK = {
        0: 0,
        1: 0xFF,
        2: 0xFFFF,
        3: 0xFFFFFF,
        4: 0xFFFFFFFF,
        5: 0xFFFFFFFFFF,
        6: 0xFFFFFFFFFFFF,
        7: 0xFFFFFFFFFFFFFF,
        8: 0xFFFFFFFFFFFFFFFF,
        }

    # Helps to sign extend the run offset. Key is the number of bytes in the
    # offset, value is the sign bit.
    SIGN_BIT = {
        0: 0,
        1: 1 << (1 * 8 - 1),
        2: 1 << (2 * 8 - 1),
        3: 1 << (3 * 8 - 1),
        4: 1 << (4 * 8 - 1),
        5: 1 << (5 * 8 - 1),
        6: 1 << (6 * 8 - 1),
        7: 1 << (7 * 8 - 1),
        8: 1 << (8 * 8 - 1),
        }

    def sign_extend(self, x, b):
        """Sign extend a value based on the number of bytes it should take."""
        m = self.SIGN_BIT[b]
        x = x & self.MASK[b]
        return (x ^ m) - m

    @property
    def data(self):
        """Returns an address space representing the data of this attribute."""
        if self.resident:
            return addrspace.BufferAddressSpace(
                data=self.obj_vm.read(
                    self.obj_offset + self.content_offset,
                    self.content_size))
        else:
            # Create a defragmented address space.
            address_space = RunListAddressSpace(
                run_list=list(self.RunList()),
                base=self.session.physical_address_space,
                session=self.session)

            return address_space

    def DecodeAttribute(self):
        if self.type == "$STANDARD_INFORMATION":
            return self.obj_profile.STANDARD_INFORMATION(
                offset=0, vm=self.data)

        elif self.type == "$FILE_NAME":
            return self.obj_profile.FILE_NAME(
                offset=0, vm=self.data)

        elif self.type == "$DATA":
            return list(self.RunList())

    def RunList(self):
        """Decodes the runlist for this attribute."""
        offset = self.obj_offset + self.runlist_offset
        run_offset = 0

        while 1:
            idx = ord(self.obj_vm.read(offset, 1))
            if idx == 0:
                return

            length_size = idx & 0xF
            run_offset_size = idx >> 4
            offset += 1

            run_length = struct.unpack("<Q", self.obj_vm.read(offset, 8))[0]
            run_length &= self.MASK[length_size]
            offset += length_size

            relative_run_offset = struct.unpack(
                "<Q", self.obj_vm.read(offset, 8))[0]

            relative_run_offset = self.sign_extend(relative_run_offset,
                                                   run_offset_size)

            run_offset += relative_run_offset
            offset += run_offset_size

            for i in range(run_offset, run_offset + run_length):
                yield i


class NTFSProfile(basic.ProfileLLP64, basic.BasicWindowsClasses):
    """A profile for the NTFS."""

    def __init__(self, **kwargs):
        super(NTFSProfile, self).__init__(**kwargs)
        self.add_overlay(ntfs_vtypes)
        self.add_classes(dict(
                NTFS_BOOT_SECTOR=NTFS_BOOT_SECTOR,
                MFT_ENTRY=MFT_ENTRY,
                NTFS_ATTRIBUTE=NTFS_ATTRIBUTE,
                ))

        # We start off with a constant mapping of attribute types. This may
        # later be updated when parsing the $AttrDef file.
        self.add_constants(ATTRIBUTE_NAMES={
                16: "$STANDARD_INFORMATION",
                32: "$ATTRIBUTE_LIST",
                48: "$FILE_NAME",
                64: "$OBJECT_ID",
                80: "$SECURITY_DESCRIPTOR",
                96: "$VOLUME_NAME",
                112: "$VOLUME_INFORMATION",
                128: "$DATA",
                144: "$INDEX_ROOT",
                160: "$INDEX_ALLOCATION",
                176: "$BITMAP",
                192: "$REPARSE_POINT",
                256: "$LOGGED_UTILITY_STREAM",
                })


class NTFS(object):
    """A class to manage the NTFS filesystem parser."""

    def __init__(self, address_space, session=None):
        self.profile = NTFSProfile(session=session)
        self.bs = self.profile.NTFS_BOOT_SECTOR(vm=address_space)

        # Check for validity of boot sector.
        self.bs.is_valid()

        session.cluster_size = self.bs.cluster_size

        # Now we search for the $DATA attribute of the $MFT file so we can
        # defragment the MFT.
        mft = self.bs.MFT[0]

        self.address_space = None

        for attribute in mft.attributes:
            if attribute.type == "$DATA":
                run_list = list(attribute.RunList())
                print run_list

                self.address_space = RunListAddressSpace(
                    run_list=run_list, base=address_space, session=session)
                break

        if self.address_space is None:
            raise NTFSParseError("Unable to locate the $MFT.")

        # The MFT is constructed over the RunListAddressSpace to reassemble the
        # fragmentation.
        self.mft = self.profile.Array(offset=0, vm=self.address_space,
                                      target="MFT_ENTRY",
                                      target_size=self.bs.mft_record_size,
                                      )



class FLS(plugin.Command):
    """List files in an NTFS image."""

    __name = "fls"

    @classmethod
    def args(cls, parser):
        super(FLS, cls).args(parser)
        parser.add_argument(
            "mfts", action=args.ArrayIntParser, nargs="+",
            help="MFT entries to list.")

    def __init__(self, mfts=None, **kwargs):
        super(FLS, self).__init__(**kwargs)
        self.mfts = mfts

    @classmethod
    def is_active(cls, session):
        return isinstance(session.profile, NTFSProfile)

    def render(self, renderer):
        ntfs = NTFS(self.session.physical_address_space,
                    session=self.session)

        for mft in self.mfts:
            for attribute in ntfs.mft[mft].attributes:
                print attribute

                decoded_attribute = attribute.DecodeAttribute()
                if decoded_attribute:
                    print decoded_attribute




# Rekall Memory Forensics
# Copyright (c) 2010, 2011, 2012 Michael Ligh <michael.ligh@mnin.org>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from rekall import obj
from rekall.plugins.windows import common
from rekall.plugins.windows import vadinfo


#--------------------------------------------------------------------------------
# vtypes
#--------------------------------------------------------------------------------

SERVICE_TYPE_FLAGS = {
            'SERVICE_KERNEL_DRIVER': 0,
            'SERVICE_FILE_SYSTEM_DRIVER': 1,
            'SERVICE_WIN32_OWN_PROCESS': 4,
            'SERVICE_WIN32_SHARE_PROCESS': 5,
            'SERVICE_INTERACTIVE_PROCESS': 8}

SERVICE_STATE_ENUM = {
            1: 'SERVICE_STOPPED',
            2: 'SERVICE_START_PENDING',
            3: 'SERVICE_STOP_PENDING',
            4: 'SERVICE_RUNNING',
            5: 'SERVICE_CONTINUE_PENDING',
            6: 'SERVICE_PAUSE_PENDING',
            7: 'SERVICE_PAUSED'}

svcscan_base_x86 = {
    '_SERVICE_HEADER': [ None, {
    'Tag': [ 0x0, ['array', 4, ['unsigned char']]],
    'ServiceRecord': [ 0xC, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_LIST_ENTRY' : [ 0x8, {
    'Blink' : [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
    'Flink' : [ 0x4, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_RECORD' : [ None, {
    'ServiceList' : [ 0x0, ['_SERVICE_LIST_ENTRY']],
    'ServiceName' : [ 0x8, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 512)]]],
    'DisplayName' : [ 0xc, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 512)]]],
    'Order' : [ 0x10, ['unsigned int']],
    'Tag' : [ 0x18, ['array', 4, ['unsigned char']]],
    'DriverName' : [ 0x24, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 256)]]],
    'ServiceProcess' : [ 0x24, ['pointer', ['_SERVICE_PROCESS']]],
    'Type' : [ 0x28, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
    'State' : [ 0x2c, ['Enumeration', dict(target='long',
                                           choices=SERVICE_STATE_ENUM)]],
    }],
    '_SERVICE_PROCESS' : [ None, {
    'BinaryPath' : [ 0x8, ['pointer', ['UnicodeString', dict(
                            encoding='utf16', length=256)]]],
    'ProcessId' : [ 0xc, ['unsigned int']],
    } ],
}

svcscan_base_x64 = {
    '_SERVICE_HEADER': [ None, {
    'Tag': [ 0x0, ['array', 4, ['unsigned char']]],
    'ServiceRecord': [ 0x10, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_LIST_ENTRY' : [ 0x8, {
    'Blink' : [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
    'Flink' : [ 0x10, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_RECORD' : [ None, {
    'ServiceList' : [ 0x0, ['_SERVICE_LIST_ENTRY']],
    'ServiceName' : [ 0x8, ['pointer', ['UnicodeString', dict(
                            encoding='utf16', length=512)]]],
    'DisplayName' : [ 0x10, ['pointer', ['UnicodeString', dict(
                            encoding='utf16', length=512)]]],
    'Order' : [ 0x18, ['unsigned int']],
    'Tag' : [ 0x20, ['array', 4, ['unsigned char']]],
    'DriverName' : [ 0x30, ['pointer', ['UnicodeString', dict(
                            encoding='utf16', length=256)]]],
    'ServiceProcess' : [ 0x30, ['pointer', ['_SERVICE_PROCESS']]],
    'Type' : [ 0x38, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
    'State' : [ 0x3C, ['Enumeration', dict(
                        target='long', choices=SERVICE_STATE_ENUM)]],
    } ],
    '_SERVICE_PROCESS': [ None, {
    'BinaryPath': [ 0x10, ['pointer', ['UnicodeString', dict(
                            encoding='utf16', length=256)]]],
    'ProcessId': [ 0x18, ['unsigned int']],
    } ],
}

#--------------------------------------------------------------------------------
# object Classes
#--------------------------------------------------------------------------------

class _SERVICE_RECORD_LEGACY(obj.Struct):
    "Service records for XP/2003 x86 and x64"

    @property
    def Binary(self):
        "Return the binary path for a service"

        # No path in memory for services that aren't running
        # (if needed, query the registry key)
        if str(self.State) != 'SERVICE_RUNNING':
            return obj.NoneObject("No path, service isn't running")

        # Depending on whether the service is for a process
        # or kernel driver, the binary path is stored differently
        if 'PROCESS' in str(self.Type):
            return self.ServiceProcess.BinaryPath.dereference()
        else:
            return self.DriverName.dereference()

    @property
    def Pid(self):
        "Return the process ID for a service"

        if str(self.State) == 'SERVICE_RUNNING':
            if 'PROCESS' in str(self.Type):
                return self.ServiceProcess.ProcessId

        return obj.NoneObject("Cannot get process ID")

    def is_valid(self):
        "Check some fields for validity"
        return (super(_SERVICE_RECORD_LEGACY, self).is_valid() and
                self.Order > 0 and self.Order < 0xFFFF)

    def traverse(self):

        rec = self # Include this object in the list
        while rec and rec.is_valid():
            yield rec
            rec = rec.ServiceList.Blink.dereference()

class _SERVICE_RECORD_RECENT(_SERVICE_RECORD_LEGACY):
    "Service records for 2008, Vista, 7 x86 and x64"

    def traverse(self):
        """Generator that walks the singly-linked list"""

        yield self # Include this object in the list

        # Make sure we dereference these pointers, or the
        # is_valid() checks will apply to the pointer and
        # not the _SERVICE_RECORD object as intended.
        rec = self.PrevEntry.dereference()
        while rec and rec.is_valid():
            yield rec
            rec = rec.PrevEntry.dereference()

class _SERVICE_HEADER(obj.Struct):
    "Service headers for 2008, Vista, 7 x86 and x64"

    def is_valid(self):
        "Check some fields for validity"
        return (obj.Struct.is_valid(self) and
                    self.ServiceRecord.is_valid() and
                    self.ServiceRecord.Order < 0xFFFF)


_SERVICE_RECORD_VISTA_X86 = {
    '_SERVICE_RECORD': [None, {
            'PrevEntry': [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
            'ServiceName': [ 0x4, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 512)]]],
            'DisplayName': [ 0x8, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 512)]]],
            'Order': [ 0xC, ['unsigned int']],
            'ServiceProcess': [ 0x1C, ['pointer', ['_SERVICE_PROCESS']]],
            'DriverName': [ 0x1C, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 256)]]],
            'Type' : [ 0x20, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
            'State': [ 0x24, ['Enumeration', dict(
                        target = 'long', choices = SERVICE_STATE_ENUM)]],
            }],
    }


_SERVICE_RECORD_VISTA_X64 = {
    '_SERVICE_RECORD': [None, {
            'PrevEntry': [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
            'ServiceName': [ 0x8, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 512)]]],
            'DisplayName': [ 0x10, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 512)]]],
            'Order': [ 0x18, ['unsigned int']],
            'ServiceProcess': [ 0x28, ['pointer', ['_SERVICE_PROCESS']]],
            'DriverName': [ 0x28, ['pointer', ['UnicodeString', dict(
                            encoding = 'utf16', length = 256)]]],
            'Type' : [ 0x30, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
            'State': [ 0x34, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
            }],
    }


class ServiceModification(obj.ProfileModification):
    """A modification for the service control manager."""

    @classmethod
    def modify(cls, profile):
        if profile.metadata("memory_model") == "32bit":
            profile.add_overlay(svcscan_base_x86)
        else:
            # 32bit Vista profiles
            profile.add_overlay(svcscan_base_x64)

        # Windows XP, 2003
        if profile.metadata("major") < 6:
            profile.add_classes({
                    '_SERVICE_RECORD': _SERVICE_RECORD_LEGACY,
                    '_SERVICE_HEADER': _SERVICE_HEADER,
                    })
            profile.add_constants(ServiceTag="sErv")

        # Vista 2008 and windows 7
        elif profile.metadata("major") >= 6:
            profile.add_classes({
                    '_SERVICE_RECORD': _SERVICE_RECORD_RECENT,
                    '_SERVICE_HEADER': _SERVICE_HEADER,
                    })
            profile.add_constants(ServiceTag="serH")

            if profile.metadata("memory_model") == "32bit":
                profile.add_overlay(_SERVICE_RECORD_VISTA_X86)
            else:
                profile.add_overlay(_SERVICE_RECORD_VISTA_X64)


#--------------------------------------------------------------------------------
# svcscan plugin
#--------------------------------------------------------------------------------

class SvcRecordScanner(vadinfo.VadScanner):
    """A scanner for the service tags."""

    def __init__(self, **kwargs):
        super(SvcRecordScanner, self).__init__(**kwargs)
        self.checks = [
            ('MultiStringFinderCheck', dict(
                    needles=[self.profile.get_constant("ServiceTag")])),
            ]
        self.tag_offset = self.profile.get_obj_offset('_SERVICE_RECORD', 'Tag')

    def scan(self, **kwargs):
        for hit in super(SvcRecordScanner, self).scan(**kwargs):
            svc_record = self.profile._SERVICE_RECORD(
                vm=self.address_space, offset=hit - self.tag_offset)

            if svc_record.is_valid():
                yield svc_record


class SvcHeaderScanner(vadinfo.VadScanner):
    """A scanner for the service tags."""

    def __init__(self, **kwargs):
        super(SvcHeaderScanner, self).__init__(**kwargs)
        self.checks = [
            ('MultiStringFinderCheck', dict(
                    needles=[self.profile.get_constant("ServiceTag")])),
            ]

        # On systems more recent than XP/2003, the serH marker doesn't
        # find *all* services, but the ones it does find have linked
        # lists to the others. We use this variable to track which
        # ones we've seen so as to not yield duplicates.
        self.records = set()

    def scan(self, **kwargs):
        for hit in super(SvcHeaderScanner, self).scan(**kwargs):
            svc_header = self.profile._SERVICE_HEADER(
                vm=self.address_space, offset=hit)

            if svc_header.is_valid():
                for record in svc_header.ServiceRecord.traverse():
                    # Since we walk the s-list backwards, if we've seen an object,
                    # then we've also seen all objects that exist before it, thus we
                    # can break at that time.
                    if record in self.records:
                        break

                    self.records.add(record)

                    yield record


class SvcScan(common.KDBGMixin, common.AbstractWindowsCommandPlugin):
    "Scan for Windows services"

    __name = "svcscan"

    def __init__(self, scan_in_kernel_address_space=False, **kwargs):
        """Scan for callbacks.

        Args:
           scan_in_kernel_address_space: If False we will use the physical
           address space for scanning, while if true we scan in the kernel
           address space.
        """
        super(SvcScan, self).__init__(**kwargs)
        self.scan_in_kernel_address_space = scan_in_kernel_address_space

        # Update the profile.
        self.profile = ServiceModification(self.profile)

    def calculate(self):
        # Get the version we're analyzing
        version = self.profile.metadatas('major', 'minor')

        pslist = self.session.plugins.pslist(proc_regex="services.exe")
        for task in pslist.filter_processes():
            # Process AS must be valid
            process_space = task.get_process_address_space()
            if process_space == None:
                continue

            # XP/2003 use the _SERVICE_RECORD object.
            if version <= (5,2):
                scanner = SvcRecordScanner(
                    task=task, process_profile=self.profile)
            else:
                # Windows Vista, 2008, and 7 use the _SERVICE_HEADER
                scanner = SvcHeaderScanner(
                    task=task, process_profile=self.profile)

            # Find all instances of the record tag
            for record in scanner.scan():
                yield record

    def render(self, renderer):
        for rec in self.calculate():
            # This can't possibly look neat in a table with columns...
            renderer.record(
                (("Offset", "offset", "{0:#x}", rec.obj_offset),
                 ("Order", "order", "{0}", rec.Order),
                 ("Process ID", "pid", "{0}", rec.Pid),
                 ("Service Name", "service", "{0}", rec.ServiceName.deref()),
                 ("Display Name", "display_name", "{0}", rec.DisplayName.deref()),
                 ("Service Type", "type", "{0}", rec.Type),
                 ("Service State", "state", "{0}", rec.State),
                 ("Binary Path", "binary_path", "{0}", rec.Binary)))




"""These are various utilities for rekall."""
# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@users.sourceforge.net>
# Michael Hale Ligh <michael.hale@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import importlib
import itertools
import re
import socket
import threading
import time


def SmartStr(string, encoding="utf8"):
    """Forces the string to be an encoded byte string."""
    if isinstance(string, unicode):
        return string.encode(encoding)

    return str(string)


def SmartUnicode(string, encoding="utf8"):
    """Forces the string into a unicode object."""
    try:
        # Allow the object to have a __unicode__ method.
        return unicode(string)
    except UnicodeError:
        return str(string).decode(encoding, "ignore")


def Hexdump(data, width=16):
    """ Hexdump function shared by various plugins """
    for offset in xrange(0, len(data), width):
        row_data = data[offset:offset + width]
        translated_data = [
            x if ord(x) < 127 and ord(x) > 32 else "." for x in row_data]
        hexdata = " ".join(["{0:02x}".format(ord(x)) for x in row_data])

        yield offset, hexdata, translated_data


def WriteHexdump(renderer, data, base=0, width=16):
    """Write the hexdump to the fd."""
    renderer.table_header([('Offset', 'offset', '[addrpad]'),
                           ('Hex', 'hex', '^' + str(width * 3)),
                           ('Data', 'data', '^' + str(width))])

    for offset, hexdata, translated_data in Hexdump(data):
        renderer.table_row(base + offset, hexdata, "".join(translated_data))


# This is a synchronize decorator.
def Synchronized(f):
    """Synchronization decorator."""

    def NewFunction(self, *args, **kw):
        if self.lock:
            with self.lock:
                return f(self, *args, **kw)
        else:
            return f(self, *args, **kw)

    return NewFunction


class Node(object):
    """An entry to a linked list."""
    next = None
    prev = None
    data = None

    def __init__(self, data):
        self.data = data

    def __str__(self):
        return "Node:" + SmartStr(self.data)

    def __repr__(self):
        return SmartStr(self)


class LinkedList(object):
    """A simple doubly linked list used for fast caches."""

    def __init__(self):
        # We are the head node.
        self.next = self.prev = self
        self.size = 0
        self.lock = threading.RLock()

    def Append(self, data):
        return self.AppendNode(Node(data))

    def AppendNode(self, node):
        self.size += 1
        last_node = self.prev

        last_node.next = node
        node.prev = last_node
        node.next = self

        return node

    def PopLeft(self):
        """Returns the head node and removes it from the list."""
        if self.next is self:
            raise IndexError("Pop from empty list.")

        first_node = self.next
        self.Unlink(first_node)
        return first_node.data

    def Pop(self):
        """Returns the tail node and removes it from the list."""
        if self.prev is self:
            raise IndexError("Pop from empty list.")

        last_node = self.tail
        self.Unlink(last_node)
        return last_node.data

    def Unlink(self, node):
        """Removes a given node from the list."""
        self.size -= 1

        node.prev.next = node.next
        node.next.prev = node.prev
        node.next = node.prev = None

    def __iter__(self):
        p = self.next
        while p is not self:
            yield p.data
            p = p.next

    def __len__(self):
        return self.size

    def __str__(self):
        p = self.next
        s = []
        while p is not self:
            s.append(str(p.data))
            p = p.next

        return "[" + ", ".join(s) + "]"


class FastStore(object):
    """This is a cache which expires objects in oldest first manner.

    This implementation first appeared in PyFlag and refined in GRR.

    This class implements an LRU cache which needs fast updates of the LRU order
    for random elements. This is implemented by using a dict for fast lookups
    and a linked list for quick deletions / insertions.
    """

    def __init__(self, max_size=10, kill_cb=None, lock=False):
        """Constructor.

        Args:
             max_size: The maximum number of objects held in cache.
             kill_cb: An optional function which will be called on each
                                object terminated from cache.
             lock: If True this cache will be thread safe.
        """
        self._age = LinkedList()
        self._hash = {}
        self._limit = max_size
        self._kill_cb = kill_cb
        self.lock = None
        if lock:
            self.lock = threading.RLock()


    @Synchronized
    def Expire(self):
        """Expires old cache entries."""
        while len(self._age) > self._limit:
            x = self._age.PopLeft()
            self.ExpireObject(x)

    @Synchronized
    def Put(self, key, item):
        """Add the object to the cache."""
        try:
            node, _ = self._hash[key]
            self._age.Unlink(node)
        except KeyError:
            pass

        node = self._age.Append(key)
        self._hash[key] = (node, item)

        self.Expire()

        return key

    @Synchronized
    def ExpireObject(self, key):
        """Expire a specific object from cache."""
        _, item = self._hash.pop(key, (None, None))

        if self._kill_cb and item is not None:
            self._kill_cb(item)

        return item

    @Synchronized
    def ExpireRegEx(self, regex):
        """Expire all the objects with the key matching the regex."""
        reg = re.compile(regex)
        for key in self._hash.keys():
            if reg.match(key):
                self.ExpireObject(key)

    @Synchronized
    def ExpirePrefix(self, prefix):
        """Expire all the objects with the key having a given prefix."""
        for key in self._hash.keys():
            if key.startswith(prefix):
                self.ExpireObject(key)

    @Synchronized
    def Get(self, key):
        """Fetch the object from cache.

        Objects may be flushed from cache at any time. Callers must always
        handle the possibility of KeyError raised here.

        Args:
            key: The key used to access the object.

        Returns:
            Cached object.

        Raises:
            KeyError: If the object is not present in the cache.
        """
        # Remove the item and put to the end of the age list
        try:
            node, item = self._hash[key]
            self._age.Unlink(node)
            self._age.AppendNode(node)
        except ValueError:
            raise KeyError(key)

        return item

    @Synchronized
    def __contains__(self, item):
        return item in self._hash

    @Synchronized
    def __getitem__(self, key):
        return self.Get(key)

    @Synchronized
    def Flush(self):
        """Flush all items from cache."""
        while self._age:
            x = self._age.PopLeft()
            self.ExpireObject(x)

        self._hash = {}

    @Synchronized
    def __getstate__(self):
        """When pickled the cache is fushed."""
        if self._kill_cb:
            raise RuntimeError("Unable to pickle a store with a kill callback.")

        self.Flush()
        return dict(max_size=self._limit)

    def __setstate__(self, state):
        self.__init__(max_size=state["max_size"])


class AgeBasedCache(FastStore):
    """A cache which removes objects after some time."""

    def __init__(self, max_age=20, **kwargs):
        super(AgeBasedCache, self).__init__(**kwargs)
        self.max_age = max_age

    def Put(self, key, item):
        super(AgeBasedCache, self).Put(key, (item, time.time()))

    def Get(self, key):
        item, timestamp = super(AgeBasedCache, self).Get(key)

        if timestamp + self.max_age > time.time():
            return item

        else:
            self.ExpireObject(key)
            raise KeyError("Item too old.")


# Compensate for Windows python not supporting socket.inet_ntop and some
# Linux systems (i.e. OpenSuSE 11.2 w/ Python 2.6) not supporting IPv6.

def inet_ntop(address_family, packed_ip):

    def inet_ntop4(packed_ip):
        if not isinstance(packed_ip, str):
            raise TypeError("must be string, not {0}".format(type(packed_ip)))

        if len(packed_ip) != 4:
            raise ValueError("invalid length of packed IP address string")

        return "{0}.{1}.{2}.{3}".format(*[ord(x) for x in packed_ip])

    def inet_ntop6(packed_ip):
        if not isinstance(packed_ip, str):
            raise TypeError("must be string, not {0}".format(type(packed_ip)))

        if len(packed_ip) != 16:
            raise ValueError("invalid length of packed IP address string")

        words = []
        for i in range(0, 16, 2):
            words.append((ord(packed_ip[i]) << 8) | ord(packed_ip[i + 1]))

        # Replace a run of 0x00s with None
        numlen = [(k, len(list(g))) for k, g in itertools.groupby(words)]
        max_zero_run = sorted(sorted(
                numlen, key=lambda x: x[1], reverse=True),
                              key=lambda x: x[0])[0]
        words = []
        for k, l in numlen:
            if (k == 0) and (l == max_zero_run[1]) and not None in words:
                words.append(None)
            else:
                for i in range(l):
                    words.append(k)

        # Handle encapsulated IPv4 addresses
        encapsulated = ""
        if (words[0] is None) and (len(words) == 3 or (
                len(words) == 4 and words[1] == 0xffff)):
            words = words[:-2]
            encapsulated = inet_ntop4(packed_ip[-4:])
        # If we start or end with None, then add an additional :
        if words[0] is None:
            words = [None] + words
        if words[-1] is None:
            words += [None]
        # Join up everything we've got using :s
        return (":".join(
                ["{0:x}".format(w) if w is not None else "" for w in words]) +
                encapsulated)

    if address_family == socket.AF_INET:
        return inet_ntop4(packed_ip)
    elif address_family == socket.AF_INET6:
        return inet_ntop6(packed_ip)
    raise socket.error("[Errno 97] Address family not supported by protocol")


def ConditionalImport(name):
    try:
        return importlib.import_module(name)
    except ImportError:
        pass


# This is only available on unix systems.
fcntl = ConditionalImport("fcntl")


class FileLock(object):
    """A self cleaning temporary directory."""

    def __init__(self, fd):
        self.fd = fd

    def __enter__(self):
        if fcntl:
            fcntl.flock(self.fd.fileno(), fcntl.LOCK_EX)
        return self.fd

    def __exit__(self, exc_type, exc_value, traceback):
        if fcntl:
            fcntl.flock(self.fd.fileno(), fcntl.LOCK_UN)


class AttributeDict(dict):
    """A dict that can be accessed via attributes."""

    def __setattr__(self, attr, value):
        try:
            # Check that the object itself has this attribute.
            object.__getattribute__(self, attr)

            return object.__setattr__(self, attr, value)
        except AttributeError:
            self.Set(attr, value)

    def Get(self, item, default=None):
        return self.get(item, default)

    def Set(self, attr, value):
        self[attr] = value

    def __getattr__(self, attr):
        return self.get(attr)

    def __dir__(self):
        return sorted(self)


def FormatIPAddress(family, value):
    """Formats a value as an ascii IP address determined by family."""
    return socket.inet_ntop(
        getattr(socket, str(family)),
        value.obj_vm.read(value.obj_offset, value.size()))

def ntoh(value):
    size = value.size()
    if size == 2:
        return socket.ntohs(value.v())
    elif size == 4:
        return socket.ntohl(value.v())

    from rekall import obj
    return obj.NoneObject("Not a valid integer")



# Import and register the core plugins

from rekall.plugins import addrspaces
from rekall.plugins import core
from rekall.plugins import darwin
from rekall.plugins import guess_profile
from rekall.plugins import imagecopy
from rekall.plugins import linux
from rekall.plugins import overlays
from rekall.plugins import tools
from rekall.plugins import windows



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

# The code in this directory is based on original code and algorithms by Andrew
# Case (atcuno@gmail.com).
__author__ = "Michael Cohen <scudette@google.com>"

import logging
import re

from rekall import args
from rekall import obj
from rekall import plugin
from rekall import scan
from rekall import utils

from rekall.plugins import core
from rekall.plugins.addrspaces import amd64


LOW_4GB_MASK = 0x00000000FFFFFFFF

def ID_MAP_VTOP(x):
    return x & LOW_4GB_MASK


class AbstractDarwinCommandPlugin(plugin.PhysicalASMixin,
                                 plugin.ProfileCommand):
    """A base class for all darwin based plugins."""
    __abstract = True

    @classmethod
    def is_active(cls, config):
        """We are only active if the profile is darwin."""
        return (getattr(config.profile, "_md_os", None) == 'darwin' and
                plugin.Command.is_active(config))


class CatfishScanner(scan.BaseScanner):
    checks = [
        ("StringCheck", dict(needle="Catfish \x00\x00"))
        ]



class DarwinFindDTB(AbstractDarwinCommandPlugin):
    """A scanner for DTB values on the Darwin kernel.

    For darwin, the dtb values are taken directly from the symbol file.

    This one plugin handles both 32 and 64 bits.
    """

    __name = "find_dtb"

    def dtb_hits(self):
        """Tries to locate the DTB."""
        if self.profile.get_constant("_BootPML4"):
            return self._dtb_hits_m_lion()
        else:
            return self._dtb_hits_pre_m_lion()

    def _dtb_hits_pre_m_lion(self):
        if self.profile.metadata("memory_model") == "32bit":
            result = self.profile.get_constant("_IdlePDPT")

            # Since the DTB must be page aligned, if this is not, it is probably
            # a pointer to the real DTB.
            if result % 0x1000:
                result = self.profile.get_constant_object(
                    "_IdlePDPT", "unsigned int")

            yield result, None
        else:
            result = self.profile.get_constant("_IdlePML4")
            if result > 0xffffff8000000000:
                result -= 0xffffff8000000000

            yield result, None

    def _dtb_hits_m_lion(self):
        """Get DTB on Mountain Lion kernels.

        From 10.8 Onwards, OSX implements Kernsl ASLR as described here:
        http://essay.utwente.nl/62852/1/Thesis_Daan_Keuper.pdf (3.4).

        This essentially addes a constant vm_kernel_slide to all kernel
        addresses:

        http://www.opensource.apple.com/source/xnu/xnu-2422.1.72/osfmk/mach/vm_param.h

        The Catfish trick is very similar to how we find KDBG on windows. There
        is a known signature for the lowGlo struct which we can find in memory,
        but there is also an exported symbol for this. The difference is
        therefore the slide value (vm_kernel_slide).

        http://www.opensource.apple.com/source/xnu/xnu-2422.1.72/osfmk/x86_64/lowmem_vectors.c

        lowglo lowGlo __attribute__ ((aligned(PAGE_SIZE))) = {
           .lgVerCode= { 'C','a','t','f','i','s','h',' ' },
        ....

        Note that we also allow the user to specify vm_kernel_slide in the
        session.

        #define LOW_4GB_MASK((vm_offset_t)0x00000000FFFFFFFFUL)

        At xnu-2422.1.72/osfmk/i386/pmap.h:
        #define ID_MAP_VTOP(x) ((void *)(((uint64_t)(x)) & LOW_4GB_MASK))

        xnu-2422.1.72/osfmk/x86_64/pmap.c:
        kernel_pmap->pm_cr3 = (uintptr_t)ID_MAP_VTOP(IdlePML4);

        """
        lowGlo = ID_MAP_VTOP(self.profile.get_constant("_lowGlo"))
        vm_kernel_slide = self.session.GetParameter("vm_kernel_slide")
        if vm_kernel_slide is None:
            for hit in CatfishScanner(
                address_space=self.physical_address_space,
                session=self.session).scan():

                # From this point on, the profile will automatically slide
                # constants by this amount.
                vm_kernel_slide = hit - lowGlo
                self.session.StoreParameter(
                    "vm_kernel_slide", int(vm_kernel_slide))

                bootpml4 = ID_MAP_VTOP(self.profile.get_constant("_BootPML4"))
                boot_as = amd64.AMD64PagedMemory(
                    base=self.physical_address_space, dtb=bootpml4)

                idlepml4_addr = ID_MAP_VTOP(
                    self.profile.get_constant("_IdlePML4"))

                idlepml4 = self.profile.Object(
                    "unsigned int", offset=idlepml4_addr, vm=boot_as)

                if idlepml4:
                    yield idlepml4, None

    def verify_address_space(self, address_space=None, **_):
        # Check the os version symbol using this address space.
        return "Darwin" == self.profile.get_constant_object(
            "_version",
            target="String",
            target_args=dict(length=6),
            vm=address_space)

    def render(self, renderer):
        renderer.table_header([("DTB", "dtv", "[addrpad]"),
                               ("Valid", "valid", "")])

        for dtb, _ in self.dtb_hits():
            address_space = core.GetAddressSpaceImplementation(self.profile)(
                session=self.session, base=self.physical_address_space, dtb=dtb)

            renderer.table_row(
                dtb, self.verify_address_space(address_space=address_space))


class DarwinPlugin(plugin.KernelASMixin, AbstractDarwinCommandPlugin):
    """Plugin which requires the kernel Address space to be loaded."""
    __abstract = True


class DarwinProcessFilter(DarwinPlugin):
    """A class for filtering processes."""

    __abstract = True

    @classmethod
    def args(cls, parser):
        super(DarwinProcessFilter, cls).args(parser)
        parser.add_argument("--pid",
                            action=args.ArrayIntParser, nargs="+",
                            help="One or more pids of processes to select.")

        parser.add_argument("--proc_regex", default=None,
                            help="A regex to select a process by name.")

        parser.add_argument("--phys_proc",
                            action=args.ArrayIntParser, nargs="+",
                            help="Physical addresses of proc structs.")

        parser.add_argument("--proc", action=args.ArrayIntParser, nargs="+",
                            help="Kernel addresses of proc structs.")

        parser.add_argument("--first", action=args.IntParser,
                            help="Kernel addresses of first proc to start "
                            "following.")

        parser.add_argument(
            "--method", choices=cls.METHODS, nargs="+",
            help="Method to list processes (Default uses all methods).")


    def __init__(self, pid=None, proc_regex=None, phys_proc=None, proc=None,
                 first=None, method=None, **kwargs):
        """Filters processes by parameters.

        Args:
           phys_proc_struct: One or more proc structs or offsets defined in
              the physical AS.

           pids: A list of pids.
           pid: A single pid.
        """
        super(DarwinProcessFilter, self).__init__(**kwargs)
        self.methods = method or self.METHODS

        if isinstance(phys_proc, (int, long)):
            phys_proc = [phys_proc]
        elif phys_proc is None:
            phys_proc = []

        if isinstance(proc, (int, long)):
            proc = [proc]
        elif isinstance(proc, obj.Struct):
            proc = [proc.obj_offset]
        elif proc is None:
            proc = []

        self.phys_proc = phys_proc
        self.proc = proc

        pids = []
        if isinstance(pid, list):
            pids.extend(pid)

        elif isinstance(pid, (int, long)):
            pids.append(pid)

        if self.session.pid and not pid:
            pids.append(self.session.pid)

        self.pids = pids
        self.proc_regex_text = proc_regex
        if isinstance(proc_regex, basestring):
            proc_regex = re.compile(proc_regex, re.I)

        self.proc_regex = proc_regex

        # Without a specified proc head, we use the proclist from _allproc
        # constant.
        if not first:
            first = self.profile.get_constant_object(
                "_allproc", target="proclist").lh_first

        self.first = first

        # Sometimes its important to know if any filtering is specified at all.
        self.filtering_requested = (self.pids or self.proc_regex or
                                    self.phys_proc or self.proc)

    def list_using_allproc(self):
        """List all processes by following the _allproc list head."""
        result = set(self.first.p_list)
        return result

    def list_using_tasks(self):
        """List processes using the processor tasks queue.


        See
        /osfmk/kern/processor.c (processor_set_things)
        """
        seen = set()

        tasks = self.profile.get_constant_object(
            "_tasks",
            target="queue_entry",
            vm=self.kernel_address_space)

        for task in tasks.list_of_type("task", "tasks"):
            proc = task.bsd_info.deref()
            if proc:
                seen.add(proc)

        return seen

    def list_using_pgrp_hash(self):
        """Process groups are organized in a hash chain.

        xnu-1699.26.8/bsd/sys/proc_internal.h
        """
        seen = set()

        # Note that _pgrphash is initialized through:

        # xnu-1699.26.8/bsd/kern/kern_proc.c:195
        # hashinit(int elements, int type, u_long *hashmask)

        # /xnu-1699.26.8/bsd/kern/kern_subr.c: 327
        # hashinit(int elements, int type, u_long *hashmask) {
        #    ...
        # *hashmask = hashsize - 1;

        # Hence the value in _pgrphash is one less than the size of the hash
        # table.
        pgr_hash_table = self.profile.get_constant_object(
            "_pgrphashtbl",
            target="Pointer",
            target_args=dict(
                target="Array",
                target_args=dict(
                    target="pgrphashhead",
                    count=self.profile.get_constant_object(
                        "_pgrphash", "unsigned long") + 1
                    )
                )
            )

        for slot in pgr_hash_table.deref():
            for pgrp in slot.lh_first.walk_list("pg_hash.le_next"):
                for proc in pgrp.pg_members.lh_first.walk_list(
                    "p_pglist.le_next"):
                    seen.add(proc)

        return seen

    def list_using_pid_hash(self):
        """Lists processes using pid hash tables.

        xnu-1699.26.8/bsd/kern/kern_proc.c:834:
        pfind_locked(pid_t pid)
        """
        seen = set()

        # Note that _pidhash is initialized through:

        # xnu-1699.26.8/bsd/kern/kern_proc.c:194
        # pidhashtbl = hashinit(maxproc / 4, M_PROC, &pidhash);

        # /xnu-1699.26.8/bsd/kern/kern_subr.c: 327
        # hashinit(int elements, int type, u_long *hashmask) {
        #    ...
        # *hashmask = hashsize - 1;

        # Hence the value in pidhash is one less than the size of the hash
        # table.
        pid_hash_table = self.profile.get_constant_object(
            "_pidhashtbl",
            target="Pointer",
            target_args=dict(
                target="Array",
                target_args=dict(
                    target="pidhashhead",
                    count=self.profile.get_constant_object(
                        "_pidhash", "unsigned long") + 1
                    )
                )
            )

        for plist in pid_hash_table.deref():
            for proc in plist.lh_first.walk_list("p_hash.le_next"):
                if proc:
                    seen.add(proc)

        return seen

    def list_procs(self):
        """Uses a few methods to list the procs."""
        seen = set()

        for k, handler in self.METHODS.items():
            if k in self.methods:
                result = handler(self)
                logging.debug("Listed %s processes using %s", len(result), k)
                seen.update(result)

        # Sort by pid so that the output ordering remains stable.
        return sorted(seen, key=lambda x: x.p_pid)

    def filter_processes(self):
        """Filters proc list using phys_proc and pids lists."""
        # No filtering required:
        if not self.filtering_requested:
            for proc in self.list_procs():
                yield proc
        else:
            # We need to filter by phys_proc
            for offset in self.phys_proc:
                yield self.virtual_process_from_physical_offset(offset)

            for offset in self.proc:
                yield self.profile.proc(vm=self.kernel_address_space,
                                        offset=int(offset))

            # We need to filter by pids
            for proc in self.list_procs():
                if int(proc.p_pid) in self.pids:
                    yield proc

                elif self.proc_regex and self.proc_regex.match(
                    utils.SmartUnicode(proc.p_comm)):
                    yield proc


    def virtual_process_from_physical_offset(self, physical_offset):
        """Tries to return an proc in virtual space from a physical offset.

        We do this by reflecting off the list elements.

        Args:
           physical_offset: The physcial offset of the process.

        Returns:
           an _PROC object or a NoneObject on failure.
        """
        physical_proc = self.profile.eprocess(offset=int(physical_offset),
                                              vm=self.kernel_address_space.base)

        # We cast our list entry in the kernel AS by following Flink into the
        # kernel AS and then the Blink. Note the address space switch upon
        # dereferencing the pointer.
        our_list_entry = physical_proc.procs.next.dereference(
            vm=self.kernel_address_space).prev.dereference()

        # Now we get the proc_struct object from the list entry.
        return our_list_entry.dereference_as("proc_struct", "procs")


    METHODS = {
        "allproc": list_using_allproc,
        "tasks": list_using_tasks,
        "pgrphash": list_using_pgrp_hash,
        "pidhash": list_using_pid_hash,
        }



class HeapScannerMixIn(object):
    """A mixin for converting a scanner into a heap only scanner."""
    def scan(self, **_):
        for vma in self.task.mm.mmap.walk_list("vm_next"):
            # Only use the vmas inside the heap area.
            if (vma.vm_start >= self.task.mm.start_brk or
                vma.vm_end <= self.task.mm.brk):
                for hit in super(HeapScannerMixIn, self).scan(
                    offset=vma.vm_start, maxlen=vma.vm_end-vma.vm_start):
                    yield hit


class KernelAddressCheckerMixIn(object):
    """A plugin mixin which does kernel address checks."""

    def __init__(self, **kwargs):
        super(KernelAddressCheckerMixIn, self).__init__(**kwargs)

        # We use the module plugin to help us local addresses inside kernel
        # modules.
        self.module_plugin = self.session.plugins.lsmod(session=self.session)



# Rekall Memory Forensics
# Copyright (C) 2008-2011 Volatile Systems
# Copyright (C) 2011 Jamie Levy (Gleeda) <jamie.levy@gmail.com>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Additional Authors:
# Michael Cohen <scudette@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
@author:       Jamie Levy (gleeda)
@license:      GNU General Public License 2.0 or later
@contact:      jamie.levy@gmail.com
@organization: Volatile Systems
"""
from rekall import obj
from rekall import scan
from rekall import utils
from rekall.plugins import core
from rekall.plugins.windows.registry import registry
from rekall.plugins.windows.registry import getsids

import os, datetime, ntpath


# for more information on Event Log structures see WFA 2E pg 260-263 by Harlan
# Carvey
evt_log_types = {
    'EVTLogHeader' : [ 0x30, {
        'HeaderSize' : [ 0x0, ['unsigned int']],
        'Magic' : [ 0x4, ['String', dict(length=4)]],  # LfLe

        # Offset of oldest record.
        'OffsetOldest' : [ 0x10, ['int']],

        # Offset of next record to be written.
        'OffsetNextToWrite' : [ 0x14, ['int']],
        'NextID' : [ 0x18, ['int']],  # Next event record ID.
        'OldestID' : [ 0x1c, ['int']], # Oldest event record ID.

        # Maximum size of event record (from registry).
        'MaxSize' : [ 0x20, ['int']],

        # Retention time of records (from registry).
        'RetentionTime' : [ 0x28, ['int']],

        # Size of the record (repeat of DWORD at offset 0).
        'RecordSize' : [ 0x2c, ['int']],
    } ],

    'EVTRecordStruct' : [ 0x38, {
        'RecordLength' : [ 0x0, ['int']],
        'Magic' : [ 0x4, ['String', dict(length=4)]],  # LfLe
        'RecordNumber' : [ 0x8, ['int']],

        'TimeGenerated' : [ 0xc, ['UnixTimeStamp']],
        'TimeWritten' : [ 0x10, ['UnixTimeStamp']],

        # Specific to event source and uniquely identifies the event.
        'EventID' : [ 0x14, ['unsigned short']],
        'EventType' : [ 0x18, ['Enumeration', dict(
                        target = 'unsigned short',
                        choices = {0x01: "Error",
                                   0x02: "Warning",
                                   0x04: "Info",
                                   0x08: "Success",
                                   0x10: "Failure"})]],

        # Number of description strings in event message.
        'NumStrings' : [ 0x1a, ['unsigned short']],
        'EventCategory' : [ 0x1c, ['unsigned short']],
        'ReservedFlags' : [ 0x1e, ['unsigned short']],
        'ClosingRecordNum' : [ 0x20, ['int']],

        # Offset w/in record of description strings.
        'StringOffset' : [ 0x24, ['int']],

        # Length of SID: if 0 no SID is present.
        'SidLength' : [ 0x28, ['int']],

        # Offset w/in record to start of SID (if present).
        'SidOffset' : [ 0x2c, ['int']],

        # Length of binary data of record.
        'DataLength' : [ 0x30, ['int']],

        # Offset of data w/in record.
        'DataOffset' : [ 0x34, ['int']],

        'Source': [0x38, ['UnicodeString', dict(
                        length=lambda x: x.RecordLength)]],

        # The computer name is right after the Source.
        'Computer': [lambda x: x.Source.obj_offset + x.Source.size(),
                     ['UnicodeString', dict(
                        length=lambda x: x.RecordLength)]],

        'Sid': [lambda x: x.obj_offset + x.SidOffset.v(), ['_SID']],

        'Data':[lambda x: x.obj_offset + x.StringOffset.v(), [
                    "ListArray", dict(
                        target="UnicodeString",
                        target_args=dict(encoding="utf16"),
                        maximum_size=lambda x: x.RecordLength,
                        count=lambda x: x.NumStrings)]],
    } ],

    "_SID": [None, {
            "IdentifierAuthority": [None, ["Enumeration", dict(
                        choices={
                            "\x00\x00\x00\x00\x00\x00": "Null Authority",
                            "\x00\x00\x00\x00\x00\x01": "World Authority",
                            "\x00\x00\x00\x00\x00\x02": "Local Authority",
                            "\x00\x00\x00\x00\x00\x03": "Creator Authority",
                            "\x00\x00\x00\x00\x00\x04": "NonUnique Authority",
                            "\x00\x00\x00\x00\x00\x05": "NT Authority",
                            },
                        target="String",
                        target_args=dict(length=6, term=None)
                        )]],
            "NumericIdentifier": [0x4, ["unsigned be int"]],
            "SubAuthority": [None, ["Array", dict(
                        target="unsigned long",
                        count=lambda x: x.SubAuthorityCount)]],
            }],
    }



class _SID(obj.Struct):
    """A Pretty printing implementation of sids.

    Reference: http://www.sekchek.com/downloads/white-papers/windows-about-sids.pdf
    """
    def __str__(self):
        """Format the Sid using SDDL Notation."""
        components = [self.Revision, self.NumericIdentifier]
        components.extend(self.SubAuthority)

        result = "S-" + "-".join([str(x) for x in components])

        # Try to resolve a friendly name from the cache in the context.
        friendly_name = self.obj_context.get("sid_cache", {}).get(result)
        if friendly_name:
            result = "%s (%s)" % (result, friendly_name)

        return result


class EVTObjectTypes(obj.ProfileModification):
    """An implementation for parsing event logs."""

    @classmethod
    def modify(cls, profile):
        profile.add_overlay(evt_log_types)
        profile.add_classes(dict(_SID=_SID))


class EVTScanner(scan.BaseScanner):
    checks = [('MultiStringFinderCheck', dict(needles=["LfLe"]))]

    def scan(self, offset, maxlen=None, context=None):
        for hit in super(EVTScanner, self).scan(offset, maxlen=maxlen):
            event_offset = hit - self.profile.get_obj_offset(
                "EVTRecordStruct", "Magic")

            event = self.profile.EVTRecordStruct(
                offset=event_offset, vm=self.address_space, context=context)

            # Eliminate crazy events (between 2001 and 2017):
            if (1500000000 > event.TimeGenerated > 1000000000 and
                1500000000 > event.TimeWritten > 1000000000):
                yield event


class EvtLogs(registry.RegistryPlugin):
    """Extract Windows Event Logs (XP/2003 only)"""

    __name = "evtlogs"

    @classmethod
    def args(cls, parser):
        super(EvtLogs, cls).args(parser)
        parser.add_argument("-v", "--verbose", default=False, action="store_true",
                            help='Resolve sids to users, services etc.')

    @classmethod
    def is_active(cls, config):
        """Only active for windows XP."""
        return (super(EvtLogs, cls).is_active(config) and
                config.profile.metadata("major") == 5)

    def __init__(self, save_evt=False, verbose=False, **kwargs):
        self.save_evt = save_evt
        self.verbose = verbose
        super(EvtLogs, self).__init__(**kwargs)
        self.profile = EVTObjectTypes(self.profile)
        self.context = dict(sid_cache={})

    def FindEVTFiles(self):
        """Search for event log files in memory.

        We search for processes called 'services.exe' with a vad to and open
        file ending with '.evt'.
        """
        ps_plugin = self.get_plugin("pslist", proc_regex="services.exe")

        for task in ps_plugin.filter_processes():
            for vad in task.RealVadRoot.traverse():
                try:
                    filename = vad.ControlArea.FilePointer.FileName
                    if utils.SmartUnicode(filename).lower().endswith(".evt"):
                        yield task, vad
                except AttributeError:
                    pass

    def ScanEvents(self, vad, address_space):
        scanner = EVTScanner(profile=self.profile, address_space=address_space,
                             session=self.session)
        for event in scanner.scan(offset=vad.Start, maxlen=vad.Length,
                                  context=self.context):
            yield event

    def PrecacheSids(self):
        """Search for known sids that we can cache."""
        sid_cache = self.context["sid_cache"]
        sid_cache.update(getsids.well_known_sids)

        # Search for all known user sids.
        for hive_offset in self.hive_offsets:
            hive_address_space = registry.HiveAddressSpace(
                base=self.kernel_address_space,
                hive_addr=hive_offset, profile=self.profile)

            reg = registry.Registry(
                profile=self.profile, address_space=hive_address_space)

            # We get the user names according to the name of the diretory where
            # their profile is. This is not very accurate - should we check the
            # SAM instead?
            profiles = reg.open_key('Microsoft\\Windows NT\\CurrentVersion\\ProfileList')
            for profile in profiles.subkeys():
                path = profile.open_value("ProfileImagePath").DecodedData
                if path:
                    sid_cache[utils.SmartUnicode(profile.Name)] = utils.SmartUnicode(
                        ntpath.basename(path))

        # Search for all service sids.
        getservicesids = self.get_plugin("getservicesids")
        for sid, service_name in getservicesids.get_service_sids():
            sid_cache[sid] = "(Service: %s)" % service_name

    def render(self, renderer):
        if self.verbose:
            self.PrecacheSids()

        renderer.table_header([("TimeWritten", "timestamp", ""),
                               ("Filename", "filename", ""),
                               ("Computer", "computer", ""),
                               ("Sid", "sid", ""),
                               ("Source", "source", ""),
                               ("Event Id", "event_id", ""),
                               ("Event Type", "event_type", ""),
                               ("Message", "message", "")])

        for task, vad in self.FindEVTFiles():
            filename = ntpath.basename(
                utils.SmartUnicode(vad.ControlArea.FilePointer.FileName))

            for event in self.ScanEvents(vad, task.get_process_address_space()):
                renderer.table_row(event.TimeWritten,
                                   filename,
                                   event.Computer,
                                   event.Sid,
                                   event.Source,
                                   event.EventID,
                                   event.EventType,
                                   ";".join(repr(utils.SmartStr(x)) for x in event.Data))



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

""" This plugin contains CORE classes used by lots of other plugins """

# pylint: disable=protected-access

import logging
import re

from rekall import args
from rekall import scan
from rekall import obj
from rekall import plugin
from rekall import utils

from rekall.plugins import core


# We require both a physical AS set and a valid profile for
# AbstractWindowsCommandPlugins.

class AbstractWindowsCommandPlugin(plugin.PhysicalASMixin,
                                   plugin.ProfileCommand):
    """A base class for all windows based plugins.

    Windows based plugins require at a minimum a working profile, and a valid
    physical address space.
    """

    __abstract = True

    @classmethod
    def is_active(cls, config):
        """We are only active if the profile is windows."""
        return (getattr(config.profile, "_md_os", None) == 'windows' and
                plugin.Command.is_active(config))


class WinFindDTB(AbstractWindowsCommandPlugin):
    """A plugin to search for the Directory Table Base for windows systems.

    There are a number of ways to find the DTB:

    - Scanner method: Scans the image for a known kernel process, and read the
      DTB from its Process Environment Block (PEB).

    - Get the DTB from the KPCR structure.

    - Note that the kernel is mapped into every process's address space (with
      the exception of session space which might be different) so using any
      process's DTB from the same session will work to read kernel data
      structures. If this plugin fails, try psscan to find potential DTBs.
    """

    __name = "find_dtb"

    # We scan this many bytes at once
    SCAN_BLOCKSIZE = 1024 * 1024

    @classmethod
    def args(cls, parser):
        """Declare the command line args we need."""
        super(WinFindDTB, cls).args(parser)
        parser.add_argument("--process_name",
                            help="The name of the process to search for.")

    def __init__(self, process_name=None, **kwargs):
        """Scans the image for the Idle process.

        Args:
          process_name: The name of the process we should look for. (If we are
            looking for the kernel DTB, any kernel process will do here.)

          physical_address_space: The address space to search. If None, we use
            the session's physical_address_space.

          profile: An optional profile to use (or we use the session's).
        """
        super(WinFindDTB, self).__init__(**kwargs)

        self.process_name = process_name or "Idle"

        # This is the offset from the ImageFileName member to the start of the
        # _EPROCESS
        self.image_name_offset = self.profile.get_obj_offset(
            "_EPROCESS", "ImageFileName")

    def scan_for_process(self):
        """Scan the image for the idle process."""
        needle = self.process_name + "\x00" * (15 - len(self.process_name))
        offset = 0
        while self.physical_address_space.is_valid_address(offset):
            data = self.physical_address_space.read(offset, self.SCAN_BLOCKSIZE)
            found = 0

            while 1:
                found = data.find(needle, found + 1)
                if found >= 0:
                    # We found something that looks like the process we want.
                    self.eprocess = self.profile.Object(
                        "_EPROCESS",
                        offset=offset + found - self.image_name_offset,
                        vm=self.physical_address_space)

                    yield self.eprocess
                else:
                    break

            offset += len(data)

    def dtb_hits(self):
        for eprocess in self.scan_for_process():
            result = eprocess.Pcb.DirectoryTableBase.v()
            if result:
                yield result, eprocess

    def verify_address_space(self, eprocess, address_space):
        """Check the eprocess for sanity."""
        # In windows the DTB must be page aligned, except for PAE images where
        # its aligned to a 0x20 size.
        if not self.profile.metadata("pae") and address_space.dtb & 0xFFF != 0:
            return False

        if self.profile.metadata("pae") and address_space.dtb & 0xF != 0:
            return False

        version = self.profile.metadata("major"), self.profile.metadata("minor")
        # The test below does not work on windows 8 with the idle process.
        if version < (6, 2):
            # Reflect through the address space at ourselves. Note that the Idle
            # process is not usually in the PsActiveProcessHead list, so we use
            # the ThreadListHead instead.
            list_head = eprocess.ThreadListHead.Flink

            if list_head == 0:
                return False

            me = list_head.dereference(vm=address_space).Blink.Flink
            if me.v() != list_head.v():
                return False

        return True

    def render(self, renderer):
        renderer.table_header(
            [("_EPROCESS (P)", "physical_eprocess", "[addrpad]"),
             ("DTB", "dtv", "[addrpad]"),
             ("Valid", "valid", "")])

        for dtb, eprocess in self.dtb_hits():
            address_space = core.GetAddressSpaceImplementation(self.profile)(
                session=self.session, base=self.physical_address_space, dtb=dtb)

            renderer.table_row(
                eprocess.obj_offset, dtb,
                self.verify_address_space(eprocess, address_space))


## The following are checks for pool scanners.

class PoolTagCheck(scan.ScannerCheck):
    """ This scanner checks for the occurance of a pool tag """
    def __init__(self, tag=None, tags=None, **kwargs):
        super(PoolTagCheck, self).__init__(**kwargs)
        self.tags = tags or [tag]

        # The offset from the start of _POOL_HEADER to the tag.
        self.tag_offset = self.profile.get_obj_offset(
            "_POOL_HEADER", "PoolTag")

    def skip(self, data, offset, **_):
        nextvals = []
        for tag in self.tags:
            nextval = data.find(tag, offset + 1)
            if nextval >= 0:
                nextvals.append(nextval)

        # No tag was found
        if not nextvals:
            # Substrings are not found - skip to the end of this data buffer
            return len(data) - offset + 1

        return min(nextvals) - offset - self.tag_offset

    def check(self, offset):
        for tag in self.tags:
            # Check the tag field.
            data = self.address_space.read(offset + self.tag_offset, len(tag))
            if data == tag:
                return True


class CheckPoolSize(scan.ScannerCheck):
    """ Check pool block size """
    def __init__(self, condition=None, min_size=None, **kwargs):
        super(CheckPoolSize, self).__init__(**kwargs)
        self.condition = condition
        if min_size:
            self.condition = lambda x: x >= min_size

        self.pool_align = self.profile.constants['PoolAlignment']

    def check(self, offset):
        pool_hdr = self.profile._POOL_HEADER(
            vm=self.address_space, offset=offset)

        block_size = pool_hdr.BlockSize.v()

        return self.condition(block_size * self.pool_align)


class CheckPoolType(scan.ScannerCheck):
    """ Check the pool type """
    def __init__(self, paged=False, non_paged=False, free=False, **kwargs):
        super(CheckPoolType, self).__init__(**kwargs)
        self.non_paged = non_paged
        self.paged = paged
        self.free = free

    def check(self, offset):
        pool_hdr = self.profile._POOL_HEADER(
            vm=self.address_space, offset=offset)

        return ((self.non_paged and pool_hdr.NonPagedPool) or
                (self.free and pool_hdr.FreePool) or
                (self.paged and pool_hdr.PagedPool))


class CheckPoolIndex(scan.ScannerCheck):
    """ Checks the pool index """
    def __init__(self, value=0, **kwargs):
        super(CheckPoolIndex, self).__init__(**kwargs)
        self.value = value

    def check(self, offset):
        pool_hdr = self.profile._POOL_HEADER(
            vm=self.address_space, offset=offset)

        return pool_hdr.PoolIndex == self.value


class PoolScanner(scan.DiscontigScanner, scan.BaseScanner):
    """A scanner for pool allocations."""

    # These objects are allocated in the pool allocation.
    allocation = ['_POOL_HEADER']

    def scan(self, offset=0, maxlen=None):
        """Yields instances of _POOL_HEADER which potentially match."""

        maxlen = maxlen or self.profile.get_constant("MaxPointer")
        for hit in super(PoolScanner, self).scan(offset=offset, maxlen=maxlen):
            yield self.profile._POOL_HEADER(vm=self.address_space, offset=hit)


class PoolScannerPlugin(plugin.KernelASMixin, AbstractWindowsCommandPlugin):
    """A base class for all pool scanner plugins."""
    __abstract = True

    @classmethod
    def args(cls, parser):
        super(PoolScannerPlugin, cls).args(parser)
        parser.add_argument(
            "--scan_in_kernel", default=False, action="store_true",
            help="Scan in the kernel address space")

    def __init__(self, address_space=None, scan_in_kernel=False, **kwargs):
        """Scan the address space for pool allocations.

        Args:
          address_space: If provided we scan this address space, else we use the
          physical_address_space.

          scan_in_kernel: Scan in the kernel address space.
        """
        super(PoolScannerPlugin, self).__init__(**kwargs)
        scan_in_kernel = scan_in_kernel or self.session.scan_in_kernel
        if scan_in_kernel:
            self.address_space = address_space or self.kernel_address_space
        else:
            self.address_space = address_space or self.physical_address_space


class KDBGMixin(plugin.KernelASMixin):
    """A plugin mixin to make sure the kdbg is set correctly."""

    @classmethod
    def args(cls, parser):
        """Declare the command line args we need."""
        super(KDBGMixin, cls).args(parser)
        parser.add_argument("--kdbg", action=args.IntParser,
                            help="Location of the KDBG structure.")

    def __init__(self, kdbg=None, **kwargs):
        """Ensure there is a valid KDBG object.

        Args:
          kdbg: The location of the kernel debugger block (In the physical
             AS).
        """
        super(KDBGMixin, self).__init__(**kwargs)
        self.kdbg = kdbg or self.session.GetParameter("kdbg")

        # If the user specified the kdbg use it - even if it looks wrong!
        if self.kdbg and not isinstance(self.kdbg, obj.BaseObject):
            kdbg = self.profile._KDDEBUGGER_DATA64(
                offset=int(self.kdbg), vm=self.kernel_address_space)

            # If the user specified the kdbg use it - even if it looks wrong!
            # This allows the user to force a corrupt kdbg.
            self.kdbg = kdbg

        if self.kdbg is None:
            logging.info(
                "KDBG not provided - Rekall Memory Forensics will try to "
                "automatically scan for it now using plugin.kdbgscan.")

            for kdbg in self.session.plugins.kdbgscan(
                session=self.session).hits():
                # Just return the first one
                logging.info("Found a KDBG hit %r. Hope it works. If not try "
                             "setting it manually.", kdbg)

                # Cache this for next time in the session.
                self.kdbg = kdbg
                self.session.StoreParameter("kdbg", int(kdbg))
                break

        # Allow kdbg to be an actual object.
        if isinstance(self.kdbg, obj.BaseObject):
            return

        # Or maybe its an integer representing the offset.
        elif self.kdbg:
            self.kdbg = self.profile._KDDEBUGGER_DATA64(
                offset=int(self.kdbg), vm=self.kernel_address_space)
        else:
            self.kdbg = obj.NoneObject("Could not guess kdbg offset")


class WindowsCommandPlugin(KDBGMixin, AbstractWindowsCommandPlugin):
    """A windows plugin which requires the kernel address space."""
    __abstract = True


class WinProcessFilter(KDBGMixin, AbstractWindowsCommandPlugin):
    """A class for filtering processes."""

    __abstract = True

    @classmethod
    def args(cls, parser):
        """Declare the command line args we need."""
        super(WinProcessFilter, cls).args(parser)

        parser.add_argument("--eprocess", action=args.ArrayIntParser, nargs="+",
                            help="Kernel addresses of eprocess structs.")

        parser.add_argument("--phys_eprocess",
                            action=args.ArrayIntParser, nargs="+",
                            help="Physical addresses of eprocess structs.")

        parser.add_argument("--pid",
                            action=args.ArrayIntParser, nargs="+",
                            help="One or more pids of processes to select.")

        parser.add_argument("--proc_regex", default=None,
                            help="A regex to select a process by name.")

        parser.add_argument("--eprocess_head", action=args.IntParser,
                            help="Use this as the process head. If "
                            "specified we do not use kdbg.")

    def __init__(self, eprocess=None, phys_eprocess=None, pid=None,
                 proc_regex=None, eprocess_head=None, **kwargs):
        """Filters processes by parameters.

        Args:
           physical_eprocess: One or more EPROCESS structs or offsets defined in
              the physical AS.

           pid: A single pid.

           proc_regex: A regular expression for filtering process name (using
             _EPROCESS.ImageFileName).

           eprocess_head: Use this as the start of the process listing (in case
             PsActiveProcessHead is missing).
        """
        super(WinProcessFilter, self).__init__(**kwargs)

        if isinstance(phys_eprocess, (int, long)):
            phys_eprocess = [phys_eprocess]
        elif phys_eprocess is None:
            phys_eprocess = []

        if isinstance(eprocess, (int, long)):
            eprocess = [eprocess]
        elif isinstance(eprocess, obj.Struct):
            eprocess = [eprocess.obj_offset]
        elif eprocess is None:
            eprocess = []

        self.phys_eprocess = phys_eprocess
        self.eprocess = eprocess

        pids = []
        if isinstance(pid, list):
            pids.extend(pid)

        elif isinstance(pid, (int, long)):
            pids.append(pid)

        if self.session.pid and not pid:
            pids.append(self.session.pid)

        self.pids = pids
        self.proc_regex_text = proc_regex
        if isinstance(proc_regex, basestring):
            proc_regex = re.compile(proc_regex, re.I)

        self.proc_regex = proc_regex
        self.eprocess_head = eprocess_head

        # Sometimes its important to know if any filtering is specified at all.
        self.filtering_requested = (self.pids or self.proc_regex or
                                    self.phys_eprocess or self.eprocess)

    def filter_processes(self):
        """Filters eprocess list using phys_eprocess and pids lists."""
        # No filtering required:
        if not self.filtering_requested:
            for eprocess in self.session.plugins.pslist(
                session=self.session, kdbg=self.kdbg,
                eprocess_head=self.eprocess_head).list_eprocess():
                yield eprocess
        else:
            # We need to filter by phys_eprocess
            for offset in self.phys_eprocess:
                yield self.virtual_process_from_physical_offset(offset)

            for offset in self.eprocess:
                yield self.profile._EPROCESS(vm=self.kernel_address_space,
                                             offset=int(offset))

            # We need to filter by pids
            for eprocess in self.session.plugins.pslist(
                session=self.session, kdbg=self.kdbg,
                eprocess_head=self.eprocess_head).list_eprocess():
                if int(eprocess.UniqueProcessId) in self.pids:
                    yield eprocess
                elif self.proc_regex and self.proc_regex.match(
                    utils.SmartUnicode(eprocess.ImageFileName)):
                    yield eprocess


    def virtual_process_from_physical_offset(self, physical_offset):
        """Tries to return an eprocess in virtual space from a physical offset.

        We do this by reflecting off the list elements.

        Args:
           physical_offset: The physcial offset of the process.

        Returns:
           an _EPROCESS object or a NoneObject on failure.
        """
        physical_eprocess = self.profile._EPROCESS(
            offset=int(physical_offset),
            vm=self.kernel_address_space.base)

        # We cast our list entry in the kernel AS by following Flink into the
        # kernel AS and then the Blink. Note the address space switch upon
        # dereferencing the pointer.
        our_list_entry = physical_eprocess.ActiveProcessLinks.Flink.dereference(
            vm=self.kernel_address_space).Blink.dereference()

        # Now we get the EPROCESS object from the list entry.
        return our_list_entry.dereference_as("_EPROCESS", "ActiveProcessLinks")





import logging
import unittest

from rekall import addrspace
from rekall import conf
from rekall import obj

# Import and register all the plugins.
from rekall import plugins


class ProfileTest(unittest.TestCase):
    """Test the profile implementation."""

    def setUp(self):
        # Create an address space from a buffer for testing
        self.address_space = addrspace.BufferAddressSpace(
            config=None, data="hello world" * 100)

    def testNativeTypes(self):
        # We build a simple profile with just the native types here.
        profile = obj.Profile.classes['Profile32Bits']()

        # Check that simple types work
        self.assertEqual(profile.Object("long", offset=0, vm=self.address_space),
                         0x6c6c6568)

        self.assertEqual(profile.Object("long long", offset=0, vm=self.address_space),
                         0x6f77206f6c6c6568)

    def testBitField(self):
        # We build a simple profile with just the native types here.
        profile = obj.Profile.classes['Profile32Bits']()
        profile.add_types({
                'Test': [ 0x10, {
                        'Field1': [0x00, ['BitField', dict(start_bit=0, end_bit=4)]],
                        'Field2': [0x00, ['BitField', dict(start_bit=4, end_bit=8)]],
                        }]})

        test = profile.Object("Test", offset=0, vm=self.address_space)

        bf1 = test.Field1
        bf2 = test.Field2

        self.assertEqual(bf1, 8)
        self.assertEqual(bf2, 6)

        # Check for overloaded numeric methods.
        self.assertEqual(bf1 + 1, 9)
        self.assertEqual(bf1 - 1, 7)
        self.assertEqual(bf1 + bf2, 8 + 6)
        self.assertEqual(bf1 < bf2, 8 < 6)
        self.assertEqual(bf1 > bf2, 8 > 6)
        self.assertEqual(bf1 & bf2, 8 & 6)
        self.assertEqual(bf1 ^ bf2, 8 ^ 6)
        self.assertEqual(bf1 + 6, 8 + bf2)
        self.assertEqual(bf1 < 6, 8 < bf2)
        self.assertEqual(bf1 > 6, 8 > bf2)
        self.assertEqual(bf1 & 6, 8 & bf2)
        self.assertEqual(bf1 ^ 6, 8 ^ bf2)

    def testPointer(self):
        # Create an address space from a buffer for testing
        address_space = addrspace.BufferAddressSpace(
            config=None, data="\x08\x00\x00\x00\x00\x00\x00\x00"
            "\x66\x55\x44\x33\x00\x00\x00\x00"
            "\x99\x88\x77\x66\x55\x44\x33\x22")

        vtype = {'Test': [ 0x10, {
                    # Check simple type dereferencing
                    'ptr32': [0x00, ['pointer', ['unsigned long']]],
                    'ptr64': [0x00, ['pointer', ['long long']]],

                    # Check struct dereferencing
                    'next': [0x00, ['pointer', ['Test']]],

                    # A pointer to an invalid location
                    'invalid': [0x08, ['pointer', ['long']]],

                    # A void pointer
                    'void': [0x00, ['pointer', ['void']]],
                    }]}

        # We build a simple profile with just the native types here.
        profile = obj.Profile.classes['Profile32Bits']()
        profile.add_types(vtype)

        test = profile.Object("Test", offset=0, vm=address_space)

        ptr = test.ptr32

        # Can we check the offset of members?
        self.assertEqual(profile.get_obj_offset("Test", "invalid"), 8)

        # 32 bit pointers.
        self.assertEqual(ptr.size(), 4)

        # The pointer itself is at location 0.
        self.assertEqual(ptr.obj_offset, 0)

        # But is pointing to location 8.
        self.assertEqual(ptr.v(), 8)
        self.assertEqual(int(ptr), 8)
        self.assertEqual(ptr, 8)

        # The long is the next 8 bytes.
        self.assertEqual(ptr.dereference(), 0x33445566)

        # Pointer comparison
        self.assertEqual(test.ptr32, test.ptr64)

        # We could do pointer arithmetic.
        ptr2 = ptr + 2

        # The new pointer is at location 8 (its 32 bits).
        self.assertEqual(ptr2.obj_offset, 8)

        # The pointer to long long is moving twice as fast
        self.assertEqual(test.ptr64 + 1, 0x33445566)
        self.assertEqual(test.ptr32 + 1, 0)

        # And its pointing to.
        self.assertEqual(ptr2.v(), 0x33445566)

        # Alas it cant be dereferenced.
        self.assertEqual(type(ptr2.dereference()), obj.NoneObject)
        self.assert_("invalid" in ptr2.dereference().reason)

        # This is also invalid.
        self.assertEqual(type(test.invalid.dereference()), obj.NoneObject)

        # Test nonzero.
        self.assert_(test.ptr32)

        # Note this pointer is actually zero, but it is actually valid in this AS.
        self.assert_(test.ptr32 + 1)

        # Now dereference a struct.
        ptr3 = test.next

        # This struct starts at offset 8.
        self.assertEqual(test.next.v(), 8)

        next = ptr3.dereference()

        # We get another struct from this.
        self.assertEqual(next.obj_type, "Test")

        # This new struct's ptr32 is pointing at this address now.
        self.assertEqual(next.ptr32, 0x33445566)

        # Now test 64 bit pointers.
        profile = obj.Profile.classes['Profile64Bits']()
        profile.add_types(vtype)

        test = profile.Object("Test", offset=0, vm=address_space)

        ptr = test.ptr32

        # 64 bit pointers.
        self.assertEqual(ptr.size(), 8)

        # The pointer itself is at location 0.
        self.assertEqual(ptr.obj_offset, 0)

        # But is pointing to location 8.
        self.assertEqual(ptr.v(), 8)

        # The long is the next 8 bytes.
        self.assertEqual(ptr.dereference(), 0x33445566)

        # We could do pointer arithmetic.
        ptr2 = ptr + 2

        # This will advance the pointer by 8 bytes (still pointer to long).
        self.assertEqual(ptr2.obj_offset, 8)
        self.assertEqual(ptr2, 0x33445566)

        # NOTE: We assume that long is 32 bits wide in both 64 bits and 32 bits
        # mode - which is the way windows does it. This is not the same as linux
        # which has long being the bit size in both cases.

        # Test the void pointer
        self.assertEqual(test.void, 8)
        self.assertEqual(test.void.dereference(), 0x33445566)

    def testArray(self):
        # Create an address space from a buffer for testing
        address_space = addrspace.BufferAddressSpace(
            config=None, data="abcdefghijklmnopqrstuvwxyz")

        profile = obj.Profile.classes['Profile32Bits']()
        test = profile.Object("Array", vm=address_space, offset=0,
                              target="int", count=0)

        self.assertEqual(test[0], 0x64636261)
        self.assertEqual(test[1], 0x68676665)

        # Can read past the end of the array but this returns a None object.
        self.assertEqual(test[100], None)


class WinXPProfileTests(unittest.TestCase):
    """Tests for basic profile functionality for the WinXP profile."""

    def setUp(self):
        # Create the address spaces from our test image. Note that we specify
        # stacking order precisely here.
        self.physical_address_space = addrspace.AddressSpaceFactory(
            specification="FileAddressSpace",
            filename="test_data/xp-laptop-2005-06-25.img")

        self.kernel_address_space = addrspace.AddressSpaceFactory(
            specification="FileAddressSpace:IA32PagedMemory",
            filename="test_data/xp-laptop-2005-06-25.img",
            dtb=0x39000)


    def testWindowsXPProfile(self):
        # We build a simple profile with just the native types here.
        profile = obj.Profile.classes['WinXPSP2x86']()

        # There is an _EPROCESS in the physical AS at this physical offset
        eprocess = profile.Object("_EPROCESS", offset=0x01343790,
                                  vm=self.physical_address_space)

        self.assertEqual(eprocess.ImageFileName, 'mqtgsvc.exe')

        # This is basically a Poor man's pslist - just follow the
        # _EPROCESS.PsActiveList around (see filescan.py).

        # First find the virtual address of the next process by reflecting
        # through the kernel AS.
        list_entry = eprocess.ThreadListHead.Flink.dereference_as(
            '_LIST_ENTRY', vm=self.kernel_address_space).Blink.dereference()

        # Take us back to the _EPROCESS offset by subtracting the _LIST_ENTRY
        # offset.
        list_entry_offset = profile.get_obj_offset('_EPROCESS', 'ThreadListHead')

        # This is now the virtual offset of the _EPROCESS.
        kernel_eprocess_offset = list_entry.obj_offset - list_entry_offset

        # Now lets get the _EPROCESS from the kernel AS this time.
        eprocess = profile.Object("_EPROCESS", offset=kernel_eprocess_offset,
                                  vm=self.kernel_address_space)

        # Lets get all the process names now. Note that we will be finding
        # PsActiveProcessHead so one name will be screwed up (because thats not
        # an _EPROCESS as all).
        names = []
        for p in eprocess.ActiveProcessLinks:
            names.append(p.ImageFileName)

        expected_names = ['alg.exe', 'wuauclt.exe','firefox.exe','PluckSvr.exe',
                          'iexplore.exe','PluckTray.exe','PluckUpdater.ex',
                          'PluckUpdater.ex','PluckTray.exe','cmd.exe','wmiprvse.exe',
                          'PluckTray.exe','dd.exe', None,'System','smss.exe','csrss.exe',
                          'winlogon.exe','services.exe','lsass.exe','svchost.exe',
                          'svchost.exe','svchost.exe','Smc.exe','svchost.exe',
                          'svchost.exe','spoolsv.exe','ssonsvr.exe','explorer.exe',
                          'Directcd.exe','TaskSwitch.exe','Fast.exe','VPTray.exe',
                          'atiptaxx.exe','jusched.exe','EM_EXEC.EXE','ati2evxx.exe',
                          'Crypserv.exe','DefWatch.exe','msdtc.exe','Rtvscan.exe',
                          'tcpsvcs.exe','snmp.exe','svchost.exe','wdfmgr.exe',
                          'Fast.exe','mqsvc.exe','']

        for i, (name, expected_name) in enumerate(zip(names, expected_names)):
            if expected_name is not None:
                self.assertEqual(name, expected_name)

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from rekall.plugins.overlays.windows import tcpip_vtypes
from rekall.plugins.windows import common

protos = {
          0:"HOPOPT",
          1:"ICMP",
          2:"IGMP",
          3:"GGP",
          4:"IPv4",
          5:"ST",
          6:"TCP",
          7:"CBT",
          8:"EGP",
          9:"IGP",
          10:"BBN-RCC-MON",
          11:"NVP-II",
          12:"PUP",
          13:"ARGUS",
          14:"EMCON",
          15:"XNET",
          16:"CHAOS",
          17:"UDP",
          18:"MUX",
          19:"DCN-MEAS",
          20:"HMP",
          21:"PRM",
          22:"XNS-IDP",
          23:"TRUNK-1",
          24:"TRUNK-2",
          25:"LEAF-1",
          26:"LEAF-2",
          27:"RDP",
          28:"IRTP",
          29:"ISO-TP4",
          30:"NETBLT",
          31:"MFE-NSP",
          32:"MERIT-INP",
          33:"DCCP",
          34:"3PC",
          35:"IDPR",
          36:"XTP",
          37:"DDP",
          38:"IDPR-CMTP",
          39:"TP++",
          40:"IL",
          41:"IPv6",
          42:"SDRP",
          43:"IPv6-Route",
          44:"IPv6-Frag",
          45:"IDRP",
          46:"RSVP",
          47:"GRE",
          48:"DSR",
          49:"BNA",
          50:"ESP",
          51:"AH",
          52:"I-NLSP",
          53:"SWIPE",
          54:"NARP",
          55:"MOBILE",
          56:"TLSP",
          57:"SKIP",
          58:"IPv6-ICMP",
          59:"IPv6-NoNxt",
          60:"IPv6-Opts",
          61:"Host-interal",
          62:"CFTP",
          63:"Local Network",
          64:"SAT-EXPAK",
          65:"KRYPTOLAN",
          66:"RVD",
          67:"IPPC",
          68:"Dist-FS",
          69:"SAT-MON",
          70:"VISA",
          71:"IPCV",
          72:"CPNX",
          73:"CPHB",
          74:"WSN",
          75:"PVP",
          76:"BR-SAT-MON",
          77:"SUN-ND",
          78:"WB-MON",
          79:"WB-EXPAK",
          80:"ISO-IP",
          81:"VMTP",
          82:"SECURE-VMTP",
          83:"VINES",
          84:"TTP",
          # 84:"IPTM",
          85:"NSFNET-IGP",
          86:"DGP",
          87:"TCF",
          88:"EIGRP",
          89:"OSPFIGP",
          90:"Sprite-RPC",
          91:"LARP",
          92:"MTP",
          93:"AX.25",
          94:"IPIP",
          95:"MICP",
          96:"SCC-SP",
          97:"ETHERIP",
          98:"ENCAP",
          99:"Encryption",
          100:"GMTP",
          101:"IFMP",
          102:"PNNI",
          103:"PIM",
          104:"ARIS",
          105:"SCPS",
          106:"QNX",
          107:"A/N",
          108:"IPComp",
          109:"SNP",
          110:"Compaq-Peer",
          111:"IPX-in-IP",
          112:"VRRP",
          113:"PGM",
          114:"0-hop",
          115:"L2TP",
          116:"DDX",
          117:"IATP",
          118:"STP",
          119:"SRP",
          120:"UTI",
          121:"SMP",
          122:"SM",
          123:"PTP",
          124:"ISIS over IPv4",
          125:"FIRE",
          126:"CRTP",
          127:"CRUDP",
          128:"SSCOPMCE",
          129:"IPLT",
          130:"SPS",
          131:"PIPE",
          132:"SCTP",
          133:"FC",
          134:"RSVP-E2E-IGNORE",
          135:"Mobility Header",
          136:"UDPLite",
          137:"MPLS-in-IP",
          138:"manet",
          139:"HIP",
          140:"Shim6",
          141:"WESP",
          142:"ROHC",
          253:"Experimental",
          254:"Experimental",
          255:"Reserved",
}


# The following are found empirically by reverse engineering. We try to find the
# TCB table which matches the image. This plugin is likely to fail for other
# operating systems and versions.
module_versions_xp = {
    'MP' : {
        'TCBTableOff' : 0x497e8,
        'SizeOff' : 0x3f7c8,
        'AddrObjTableOffset' : 0x48760,
        'AddrObjTableSizeOffset' : 0x48764,
        },
    'UP' : {
        'TCBTableOff' : 0x495e8,
        'SizeOff' : 0x3f5bc,
        'AddrObjTableOffset' : 0x48560,
        'AddrObjTableSizeOffset' : 0x48564,
        },
    '2180' : {
        'TCBTableOff' : 0x493e8,
        'SizeOff' : 0x3f3b0,
        'AddrObjTableOffset'  : 0x48360,
        'AddrObjTableSizeOffset' : 0x48364,
        },
    '3244' : {
        'TCBTableOff' : 0x496E8,
        'SizeOff' : 0x3F6BC,
        'AddrObjTableOffset'  : 0x48660,
        'AddrObjTableSizeOffset' : 0x48664,
        },
    '3394': {
        'TCBTableOff': 0x49768,
        'SizeOff': 0x3F73C,
        'AddrObjTableOffset': 0x486E0,
        'AddrObjTableSizeOffset': 0x486E4,
        },
    '5625' : {
        'TCBTableOff' : 0x49ae8,
        'SizeOff' : 0x3fac8,
        'AddrObjTableOffset'  : 0x48a60,
        'AddrObjTableSizeOffset' : 0x48a64,
        },
    '2111' : {
        'TCBTableOff' : 0x49A68,
        'SizeOff' : 0x3FA48,
        'AddrObjTableOffset'  : 0x489E0,
        'AddrObjTableSizeOffset' : 0x489E4,
        },
    }

module_versions_2k3 = {
    # w2k3 sp0
    '3790' : {
        'TCBTableOff' : 0x4c6c8,
        'SizeOff' : 0x4312c,
        'AddrObjTableOffset'  : 0x4bba0,
        'AddrObjTableSizeOffset' : 0x4bba4,
        },
    # w2k3 sp1
    '1830' : {
        'TCBTableOff' : 0x4e428,
        'SizeOff' : 0x44140,
        'AddrObjTableOffset'  : 0x4d4e4,
        'AddrObjTableSizeOffset' : 0x4d4e8,
        },
    # w2k3 sp2
    '3959' : {
        'TCBTableOff' : 0x7c548,
        'SizeOff' : 0x50308,
        'AddrObjTableOffset'  : 0x5ada4,
        'AddrObjTableSizeOffset' : 0x5ada8,
        },
    # w2k3 sp2
    '4573' : {
        'TCBTableOff' : 0x7f0ac,
        'SizeOff' : 0x52328,
        'AddrObjTableOffset'  : 0x5cf04,
        'AddrObjTableSizeOffset' : 0x5cf08,
        },
    }


class Connections(common.AbstractWindowsCommandPlugin):
    """
    Print list of open connections [Windows XP Only]
    ---------------------------------------------

    This module enumerates the active connections from tcpip.sys.

    Note that if you are using a hibernated image this might not work
    because Windows closes all sockets before hibernating. You might
    find it more effective to do conscan instead.
    """

    __name = "connections"

    @classmethod
    def is_active(cls, session):
        # These only work for XP.
        return (super(Connections, cls).is_active(session) and
                session.profile.metadata("major") == 5)

    def __init__(self, **kwargs):
        """Enumerates the active connections in tcpip.sys's data structures."""
        super(Connections, self).__init__(**kwargs)

        # Install the reversed tcpip structs.
        self.profile = tcpip_vtypes.TCPIPModifications(self.profile)

    def determine_connections(self):
        """Determines active connections in tcpip.sys"""
        version = (self.profile.metadata('major', 0),
                   self.profile.metadata('minor', 0))

        if version <= (5, 1):
            module_versions = module_versions_xp
        else:
            module_versions = module_versions_2k3

        # List all the modules using the modules plugin
        modules = self.session.plugins.modules(
            session=self.session, profile=self.profile)

        for m in modules.lsmod():
            # Try to find the tcpip.sys module.
            if str(m.BaseDllName).lower() == 'tcpip.sys':

                # Try every possibility in the lookup table.
                for attempt in module_versions:
                    SizeOff = module_versions[attempt]['SizeOff']
                    TCBTableOff = module_versions[attempt]['TCBTableOff']
                    BaseAddress = int(m.DllBase)

                    table_size = self.profile.long(
                        offset=BaseAddress + SizeOff, vm=m.obj_vm)

                    table_addr = self.profile.Object(
                        "unsigned long", offset=BaseAddress + TCBTableOff,
                        vm=m.obj_vm)

                    if table_size > 0:
                        table = self.profile.Array(
                            offset=table_addr, vm=m.obj_vm,
                            count=table_size,
                            target="Pointer", target_args=dict(
                                target='_TCPT_OBJECT'))

                        if table:
                            for entry in table:
                                conn = entry.dereference()
                                seen = set()
                                while (conn.is_valid() and
                                       conn.obj_offset not in seen):
                                    yield conn
                                    seen.add(conn.obj_offset)
                                    conn = conn.Next.dereference()


    def render(self, renderer):
        renderer.table_header(
            [("Offset (V)", "offset_v", "[addrpad]"),
             ("Local Address", "local_net_address", "25"),
             ("Remote Address", "remote_net_address", "25"),
             ("Pid", "pid", ">6")
             ])

        for conn in self.determine_connections():
            offset = conn.obj_offset
            local = "{0}:{1}".format(conn.LocalIpAddress, conn.LocalPort)
            remote = "{0}:{1}".format(conn.RemoteIpAddress, conn.RemotePort)
            renderer.table_row(offset, local, remote, conn.Pid)


class Sockets(common.AbstractWindowsCommandPlugin):
    """
    Print list of open sockets. [Windows xp only]
    ---------------------------------------------

    This module enumerates the active sockets from tcpip.sys

    Note that if you are using a hibernated image this might not work
    because Windows closes all sockets before hibernating.
    """

    __name = "sockets"

    @classmethod
    def is_active(cls, session):
        # These only work for XP.
        return (super(Sockets, cls).is_active(session) and
                session.profile.metadata("major") == 5)

    def __init__(self, **kwargs):
        """Enumerates the active sockets in tcpip.sys's data structures."""
        super(Sockets, self).__init__(**kwargs)

        # Install the reversed tcpip structs.
        self.profile = tcpip_vtypes.TCPIPModifications(self.profile)

    def determine_sockets(self):
        """Determines all active sockets in tcpip.sys"""
        version = (self.profile.metadata('major', 0),
                   self.profile.metadata('minor', 0))

        if version <= (5, 1):
            module_versions = module_versions_xp
        else:
            module_versions = module_versions_2k3

        # List all the modules using the modules plugin
        modules = self.session.plugins.modules(session=self.session)
        for m in modules.lsmod():
            # Try to find the tcpip.sys module.
            if str(m.BaseDllName).lower() == 'tcpip.sys':

                # Try every possibility in the lookup table.
                for attempt in module_versions:
                    AddrObjTableSizeOffset = module_versions[attempt][
                        'AddrObjTableSizeOffset']
                    AddrObjTableOffset = module_versions[attempt][
                        'AddrObjTableOffset']
                    BaseAddress = int(m.DllBase)

                    table_size = self.profile.Object(
                        "long", offset=BaseAddress + AddrObjTableSizeOffset,
                        vm=m.obj_vm)

                    table_addr = self.profile.Object(
                        "unsigned long", offset=BaseAddress+AddrObjTableOffset,
                        vm=m.obj_vm)

                    if table_size > 0:
                        table = self.profile.Array(
                            offset=table_addr, vm=m.obj_vm,
                            count=table_size,
                            target="Pointer", target_args=dict(
                                target='_ADDRESS_OBJECT'))

                        if table:
                            for entry in table:
                                sock = entry.dereference()
                                seen = set()
                                while (sock.is_valid() and
                                       sock.obj_offset not in seen):
                                    yield sock
                                    seen.add(sock.obj_offset)
                                    sock = sock.Next.dereference()

    def render(self, renderer):
        renderer.table_header([("Offset (V)", "offset_v", "[addrpad]"),
                               ("PID", "pid", ">6"),
                               ("Port", "port", ">6"),
                               ("Proto", "protocol_number", ">6"),
                               ("Protocol", "protocol", "15"),
                               ("Address", "address", "15"),
                               ("Create Time", "socket_create_time", "")
                               ])

        for sock in self.determine_sockets():
            offset = sock.obj_offset
            protocol = sock.Protocol.v()
            if protocol > 255:
                protocol = "-"
            else:
                protocol = protos.get(protocol, "Unassigned")

            renderer.table_row(
                offset, sock.Pid, sock.LocalPort, sock.Protocol, protocol,
                sock.LocalIpAddress, sock.CreateTime)
